from typing import Optional

MIN_INDEX = 0
MAX_INDEX = 4095


def get_word(index: int) -> Optional[str]:
    """
    Get word (4096 options) based on 16 bit index for use with trackers.
    :param index: index of word between 0 and 4095
    :return: word or None if index outside bounds
    """
    if index < MIN_INDEX:
        return None
    if index > MAX_INDEX:
        return None
    return words[index]


def get_index_from_word(word: str) -> Optional[int]:
    """
    Get the index of a word (4096 options) based on 16 bit index for use with trackers.
    :param word: word
    :return: index of word or None if not found
    """
    return words.index(word) if word in words else None


words = [
    "aardvark",
    "abacus",
    "abbey",
    "abbreviation",
    "ability",
    "abolishment",
    "absence",
    "abundance",
    "academics",
    "academy",
    "accelerant",
    "accelerator",
    "accent",
    "acceptance",
    "access",
    "accessory",
    "accident",
    "accommodation",
    "accompanist",
    "accomplishment",
    "accord",
    "accordance",
    "accordion",
    "account",
    "accountability",
    "accountant",
    "accounting",
    "accuracy",
    "achievement",
    "achiever",
    "acknowledgment",
    "acorn",
    "acoustics",
    "acquaintance",
    "acquisition",
    "acre",
    "acrylic",
    "act",
    "action",
    "activation",
    "activity",
    "actor",
    "actress",
    "adaptation",
    "adapter",
    "addition",
    "address",
    "adjective",
    "adjustment",
    "administration",
    "administrator",
    "admire",
    "admission",
    "adobe",
    "adult",
    "adulthood",
    "advance",
    "advancement",
    "advantage",
    "advent",
    "adverb",
    "advertisement",
    "advertising",
    "advice",
    "adviser",
    "advocacy",
    "advocate",
    "affect",
    "affiliate",
    "affinity",
    "afoul",
    "afterlife",
    "aftermath",
    "afternoon",
    "aftershave",
    "afterthought",
    "age",
    "agency",
    "agenda",
    "agent",
    "agony",
    "agreement",
    "agriculture",
    "aid",
    "aide",
    "aim",
    "air",
    "airbag",
    "aircraft",
    "airfare",
    "airfield",
    "airline",
    "airmail",
    "airplane",
    "airport",
    "airspace",
    "albatross",
    "album",
    "alcove",
    "alder",
    "alfalfa",
    "algebra",
    "algorithm",
    "alias",
    "alibi",
    "allergist",
    "alley",
    "alliance",
    "alligator",
    "allocation",
    "allowance",
    "alloy",
    "almanac",
    "almond",
    "alpaca",
    "alpha",
    "alphabet",
    "altar",
    "alteration",
    "alternative",
    "altitude",
    "alto",
    "aluminium",
    "aluminum",
    "amazement",
    "ambassador",
    "amber",
    "ambience",
    "ambiguity",
    "ambition",
    "ambulance",
    "amendment",
    "amenity",
    "amount",
    "amusement",
    "anagram",
    "analog",
    "analogue",
    "analogy",
    "analysis",
    "analyst",
    "analytics",
    "anatomy",
    "ancestor",
    "anchovy",
    "angel",
    "angle",
    "angora",
    "anguish",
    "animal",
    "anime",
    "anise",
    "ankle",
    "anklet",
    "anniversary",
    "announcement",
    "annual",
    "anorak",
    "answer",
    "ant",
    "anteater",
    "antelope",
    "antennae",
    "anterior",
    "anthropology",
    "antibody",
    "anticipation",
    "antique",
    "antiquity",
    "antler",
    "antling",
    "anybody",
    "anyone",
    "anything",
    "anywhere",
    "apartment",
    "ape",
    "apology",
    "app",
    "apparatus",
    "apparel",
    "appeal",
    "appearance",
    "appendix",
    "appetiser",
    "appetite",
    "appetizer",
    "applause",
    "applewood",
    "appliance",
    "application",
    "appointment",
    "appreciation",
    "apprehension",
    "approach",
    "appropriation",
    "approval",
    "apricot",
    "apron",
    "aquarium",
    "aquifer",
    "arcade",
    "arch",
    "archaeology",
    "archeology",
    "archer",
    "architect",
    "architecture",
    "archives",
    "area",
    "arena",
    "argument",
    "arithmetic",
    "ark",
    "arm",
    "armadillo",
    "armchair",
    "armoire",
    "armpit",
    "armrest",
    "arrangement",
    "array",
    "arrest",
    "arrival",
    "arrow",
    "art",
    "artery",
    "arthur",
    "artichoke",
    "article",
    "artifact",
    "artificer",
    "artist",
    "ascend",
    "ascent",
    "ascot",
    "ash",
    "aside",
    "asparagus",
    "aspect",
    "asphalt",
    "assembly",
    "assertion",
    "assessment",
    "asset",
    "assignment",
    "assist",
    "assistance",
    "assistant",
    "associate",
    "association",
    "assumption",
    "assurance",
    "asterisk",
    "astronomy",
    "asymmetry",
    "atelier",
    "athlete",
    "athletics",
    "atmosphere",
    "atom",
    "atrium",
    "attachment",
    "attainment",
    "attempt",
    "attendance",
    "attendant",
    "attention",
    "attic",
    "attitude",
    "attorney",
    "attraction",
    "attribute",
    "auction",
    "audience",
    "audit",
    "auditorium",
    "aunt",
    "authentication",
    "authenticity",
    "author",
    "authorisation",
    "authority",
    "authorization",
    "auto",
    "automation",
    "automaton",
    "autumn",
    "availability",
    "avalanche",
    "avenue",
    "average",
    "avocado",
    "award",
    "awareness",
    "awe",
    "axis",
    "babe",
    "baby",
    "bachelor",
    "back",
    "backbone",
    "backburn",
    "backdrop",
    "background",
    "backpack",
    "backup",
    "backyard",
    "bacon",
    "badge",
    "badger",
    "bafflement",
    "bag",
    "bagel",
    "baggage",
    "bagpipe",
    "bail",
    "bait",
    "bake",
    "baker",
    "bakery",
    "bakeware",
    "balance",
    "balcony",
    "ball",
    "ballet",
    "balloon",
    "balloonist",
    "ballot",
    "ballpark",
    "bamboo",
    "ban",
    "banana",
    "band",
    "bandana",
    "bandanna",
    "bandwidth",
    "bangle",
    "banjo",
    "bank",
    "bankbook",
    "banker",
    "banking",
    "bankruptcy",
    "banner",
    "banquette",
    "bar",
    "barbecue",
    "barbeque",
    "barber",
    "bargain",
    "barge",
    "baritone",
    "bark",
    "barley",
    "barn",
    "barometer",
    "barrage",
    "barrel",
    "barrier",
    "base",
    "baseball",
    "baseboard",
    "baseline",
    "basement",
    "basics",
    "basil",
    "basin",
    "basis",
    "basket",
    "basketball",
    "bass",
    "bassinet",
    "bassoon",
    "bat",
    "bath",
    "bathrobe",
    "bathroom",
    "bathtub",
    "battalion",
    "batter",
    "battery",
    "batting",
    "bay",
    "bayou",
    "beach",
    "bead",
    "beak",
    "beam",
    "bean",
    "beancurd",
    "beanie",
    "beanstalk",
    "bear",
    "beauty",
    "beck",
    "bedrock",
    "bee",
    "beech",
    "beet",
    "beetle",
    "beginner",
    "beginning",
    "begonia",
    "behalf",
    "behavior",
    "behaviour",
    "behest",
    "behold",
    "being",
    "belfry",
    "belief",
    "believer",
    "bell",
    "bellows",
    "belt",
    "bench",
    "bend",
    "benefit",
    "beret",
    "berry",
    "bestseller",
    "bet",
    "beverage",
    "beyond",
    "bias",
    "bicycle",
    "bid",
    "bidder",
    "bidding",
    "bifocals",
    "bijou",
    "bike",
    "bill",
    "billboard",
    "billing",
    "billion",
    "bin",
    "binoculars",
    "biology",
    "biopsy",
    "biosphere",
    "biplane",
    "birch",
    "bird",
    "birdbath",
    "birdcage",
    "birdhouse",
    "birthday",
    "biscuit",
    "bit",
    "bitten",
    "bitter",
    "blackberry",
    "blackbird",
    "blackboard",
    "blackfish",
    "blackness",
    "blade",
    "blame",
    "blank",
    "blanket",
    "blast",
    "blazer",
    "blend",
    "blind",
    "blinker",
    "blister",
    "blizzard",
    "block",
    "blocker",
    "blog",
    "blogger",
    "bloom",
    "bloomer",
    "blossom",
    "blouse",
    "blue",
    "blueberry",
    "blush",
    "boar",
    "board",
    "boat",
    "boatload",
    "boatyard",
    "bob",
    "bobcat",
    "bog",
    "bolero",
    "bolt",
    "bond",
    "bonding",
    "bondsman",
    "bonfire",
    "bongo",
    "bonnet",
    "bonsai",
    "bonus",
    "book",
    "bookcase",
    "bookend",
    "booking",
    "booklet",
    "bookmark",
    "boolean",
    "boom",
    "boon",
    "boost",
    "booster",
    "boot",
    "border",
    "bore",
    "borrower",
    "borrowing",
    "boss",
    "botany",
    "bother",
    "bottle",
    "bottling",
    "bough",
    "boulder",
    "boulevard",
    "boundary",
    "bouquet",
    "bout",
    "boutique",
    "bow",
    "bower",
    "bowl",
    "bowler",
    "bowling",
    "bowtie",
    "boxer",
    "boxspring",
    "boy",
    "boycott",
    "boyfriend",
    "boyhood",
    "boysenberry",
    "brace",
    "bracelet",
    "bracket",
    "brain",
    "brake",
    "bran",
    "branch",
    "brand",
    "brass",
    "bratwurst",
    "bread",
    "breadcrumb",
    "breadfruit",
    "break",
    "breakdown",
    "breakfast",
    "breakpoint",
    "breakthrough",
    "breath",
    "breeze",
    "brewer",
    "bribery",
    "brick",
    "bricklaying",
    "bride",
    "bridge",
    "brief",
    "briefing",
    "briefly",
    "brilliant",
    "brink",
    "brisket",
    "broad",
    "broadcast",
    "broccoli",
    "brochure",
    "broiler",
    "broker",
    "bronco",
    "bronze",
    "brooch",
    "brood",
    "brook",
    "broom",
    "brother",
    "brow",
    "brownie",
    "browser",
    "browsing",
    "brunch",
    "brush",
    "brushfire",
    "brushing",
    "bubble",
    "buck",
    "bucket",
    "buckle",
    "buckwheat",
    "buddy",
    "budget",
    "buffalo",
    "buffer",
    "buffet",
    "bug",
    "buggy",
    "bugle",
    "builder",
    "building",
    "bulb",
    "bulk",
    "bull",
    "bulldozer",
    "bullet",
    "bun",
    "bunch",
    "bungalow",
    "bunghole",
    "bunkhouse",
    "burden",
    "bureau",
    "burglar",
    "burial",
    "burn",
    "burning",
    "burrito",
    "burrow",
    "burst",
    "bus",
    "business",
    "businessman",
    "bustle",
    "butane",
    "butler",
    "butter",
    "butterfly",
    "button",
    "buy",
    "buyer",
    "buying",
    "buzzard",
    "cabana",
    "cabbage",
    "cabin",
    "cabinet",
    "cable",
    "caboose",
    "cacao",
    "cactus",
    "caddy",
    "cadet",
    "cafe",
    "caffeine",
    "caftan",
    "cage",
    "cake",
    "calculation",
    "calculator",
    "calculus",
    "calendar",
    "calf",
    "calico",
    "call",
    "calm",
    "calorie",
    "camel",
    "cameo",
    "camera",
    "camp",
    "campaign",
    "campaigning",
    "campanile",
    "camper",
    "campus",
    "can",
    "canal",
    "candelabra",
    "candidacy",
    "candidate",
    "candle",
    "candy",
    "cane",
    "cannon",
    "canoe",
    "canon",
    "canopy",
    "cantaloupe",
    "canteen",
    "canvas",
    "cap",
    "capability",
    "capacity",
    "cape",
    "caper",
    "capital",
    "capitulation",
    "capon",
    "cappuccino",
    "captain",
    "caption",
    "captor",
    "car",
    "caramel",
    "caravan",
    "carbon",
    "card",
    "cardboard",
    "cardigan",
    "care",
    "career",
    "cargo",
    "caribou",
    "carload",
    "carnation",
    "carnival",
    "carol",
    "carp",
    "carpenter",
    "carpet",
    "carpeting",
    "carport",
    "carriage",
    "carrier",
    "carrot",
    "carry",
    "cart",
    "cartel",
    "carter",
    "cartload",
    "cartoon",
    "cartridge",
    "carving",
    "cascade",
    "case",
    "casement",
    "cash",
    "cashew",
    "cashier",
    "casino",
    "casserole",
    "cassock",
    "cast",
    "castanet",
    "castle",
    "cat",
    "catacomb",
    "catalogue",
    "catalyst",
    "catamaran",
    "catastrophe",
    "catch",
    "catcher",
    "category",
    "caterpillar",
    "cation",
    "catsup",
    "cattle",
    "cauliflower",
    "causal",
    "cause",
    "causeway",
    "caution",
    "cave",
    "caviar",
    "cayenne",
    "ceiling",
    "celebration",
    "celebrity",
    "celeriac",
    "celery",
    "cell",
    "cellar",
    "cello",
    "celsius",
    "cement",
    "census",
    "cent",
    "center",
    "centimeter",
    "centre",
    "centurion",
    "century",
    "ceramic",
    "ceramics",
    "cereal",
    "ceremony",
    "certainty",
    "certificate",
    "certification",
    "chafe",
    "chain",
    "chainstay",
    "chair",
    "chairlift",
    "chairman",
    "chairperson",
    "chaise",
    "chalet",
    "chalice",
    "chalk",
    "challenge",
    "chamber",
    "champagne",
    "champion",
    "championship",
    "chance",
    "chandelier",
    "change",
    "channel",
    "chaos",
    "chap",
    "chapel",
    "chapter",
    "character",
    "chard",
    "charge",
    "charger",
    "charity",
    "charlatan",
    "charm",
    "chart",
    "charter",
    "chasm",
    "chassis",
    "chateau",
    "chatter",
    "check",
    "checkbook",
    "checking",
    "checkout",
    "checkroom",
    "cheddar",
    "cheek",
    "cheer",
    "cheese",
    "cheesecake",
    "cheetah",
    "chef",
    "chem",
    "chemical",
    "chemistry",
    "cherry",
    "chess",
    "chestnut",
    "chicken",
    "chicory",
    "chief",
    "child",
    "childhood",
    "chili",
    "chill",
    "chime",
    "chimpanzee",
    "chin",
    "chino",
    "chip",
    "chipmunk",
    "chivalry",
    "chive",
    "chives",
    "chocolate",
    "choice",
    "choir",
    "choker",
    "choosing",
    "chop",
    "chops",
    "chopstick",
    "chopsticks",
    "chord",
    "chorus",
    "chow",
    "chowder",
    "chrome",
    "chronicle",
    "chrysalis",
    "chub",
    "chuck",
    "chug",
    "churn",
    "chutney",
    "cicada",
    "cilantro",
    "cinder",
    "cinema",
    "cinnamon",
    "circadian",
    "circle",
    "circuit",
    "circulation",
    "circumference",
    "circumstance",
    "cirrus",
    "citizen",
    "citizenship",
    "citron",
    "citrus",
    "city",
    "civilian",
    "civilisation",
    "civilization",
    "claim",
    "clamp",
    "clan",
    "clank",
    "clapboard",
    "clarification",
    "clarinet",
    "clarity",
    "clasp",
    "class",
    "classic",
    "classmate",
    "classroom",
    "clause",
    "clave",
    "clavicle",
    "clavier",
    "claw",
    "clay",
    "cleaner",
    "clearance",
    "clearing",
    "cleat",
    "clef",
    "cleft",
    "clerk",
    "click",
    "client",
    "cliff",
    "climate",
    "climb",
    "clinic",
    "clip",
    "clipboard",
    "clipper",
    "cloak",
    "cloakroom",
    "clock",
    "clockwork",
    "clogs",
    "clone",
    "close",
    "closet",
    "closing",
    "closure",
    "cloth",
    "clothes",
    "clothing",
    "cloud",
    "cloudburst",
    "clove",
    "clover",
    "cloves",
    "club",
    "clue",
    "cluster",
    "clutch",
    "coach",
    "coal",
    "coalition",
    "coast",
    "coaster",
    "coat",
    "cob",
    "cobbler",
    "cobweb",
    "cocktail",
    "cocoa",
    "coconut",
    "cod",
    "code",
    "codepage",
    "codling",
    "codon",
    "cofactor",
    "coffee",
    "coffin",
    "cohesion",
    "cohort",
    "coil",
    "coin",
    "coincidence",
    "coinsurance",
    "cold",
    "coleslaw",
    "coliseum",
    "collaboration",
    "collagen",
    "collapse",
    "collar",
    "collard",
    "collateral",
    "colleague",
    "collection",
    "collector",
    "college",
    "collision",
    "colloquy",
    "colony",
    "color",
    "colt",
    "column",
    "columnist",
    "comb",
    "combination",
    "combine",
    "comeback",
    "comedy",
    "comfort",
    "comfortable",
    "comic",
    "comics",
    "comma",
    "command",
    "commander",
    "comment",
    "commerce",
    "commercial",
    "commission",
    "commitment",
    "committee",
    "commodity",
    "common",
    "commonsense",
    "commotion",
    "communicant",
    "communication",
    "community",
    "commuter",
    "company",
    "comparison",
    "compass",
    "compassion",
    "compassionate",
    "compensation",
    "competence",
    "competition",
    "competitor",
    "complaint",
    "complement",
    "completion",
    "complex",
    "complexity",
    "compliance",
    "complication",
    "complicity",
    "compliment",
    "component",
    "comportment",
    "composer",
    "composite",
    "composition",
    "compost",
    "comprehension",
    "compress",
    "compromise",
    "compulsion",
    "computer",
    "con",
    "concentrate",
    "concept",
    "conception",
    "concern",
    "concert",
    "conclusion",
    "concrete",
    "condition",
    "conditioner",
    "condominium",
    "condor",
    "conduct",
    "conductor",
    "cone",
    "conference",
    "confidence",
    "configuration",
    "conflict",
    "confusion",
    "conga",
    "congo",
    "congress",
    "congressman",
    "conifer",
    "connection",
    "connotation",
    "conscience",
    "consciousness",
    "consensus",
    "consent",
    "consequence",
    "conservation",
    "consideration",
    "consignment",
    "consist",
    "consistency",
    "console",
    "consonant",
    "constant",
    "constellation",
    "constitution",
    "constraint",
    "construction",
    "consul",
    "consulate",
    "consulting",
    "consumer",
    "consumption",
    "contact",
    "container",
    "content",
    "contention",
    "contest",
    "context",
    "continent",
    "continuity",
    "contour",
    "contract",
    "contractor",
    "contrary",
    "contrast",
    "contribution",
    "contributor",
    "control",
    "controller",
    "convection",
    "convenience",
    "convention",
    "conversation",
    "conversion",
    "convert",
    "convertible",
    "cook",
    "cookbook",
    "cookie",
    "cooking",
    "cooperation",
    "coordination",
    "coordinator",
    "cope",
    "copper",
    "copy",
    "copying",
    "copyright",
    "copywriter",
    "coral",
    "cord",
    "corduroy",
    "core",
    "cork",
    "corn",
    "corner",
    "cornerstone",
    "cornflakes",
    "cornmeal",
    "corporal",
    "corporation",
    "corral",
    "corridor",
    "corsage",
    "cosset",
    "cost",
    "costume",
    "cot",
    "cottage",
    "cotton",
    "couch",
    "cough",
    "council",
    "councilor",
    "counsel",
    "counseling",
    "counselling",
    "counsellor",
    "counselor",
    "count",
    "counter",
    "counterpart",
    "countess",
    "country",
    "countryside",
    "county",
    "couple",
    "coupon",
    "courage",
    "court",
    "courthouse",
    "courtroom",
    "cousin",
    "cover",
    "coverage",
    "coverall",
    "cow",
    "cowbell",
    "coyote",
    "crackers",
    "cradle",
    "craft",
    "craftsman",
    "cranberry",
    "crane",
    "cranky",
    "crash",
    "crate",
    "cravat",
    "craw",
    "crawdad",
    "crayfish",
    "crayon",
    "creation",
    "creative",
    "creativity",
    "creator",
    "creature",
    "credential",
    "credenza",
    "credibility",
    "credit",
    "creditor",
    "creek",
    "crepe",
    "crest",
    "crew",
    "crewman",
    "crewmate",
    "crewmember",
    "crewmen",
    "crib",
    "cribbage",
    "cricket",
    "crime",
    "criminal",
    "crisis",
    "crisp",
    "criteria",
    "criterion",
    "critic",
    "criticism",
    "crocodile",
    "crocus",
    "croissant",
    "crook",
    "crop",
    "cross",
    "croup",
    "crow",
    "crowd",
    "crown",
    "cruise",
    "crumb",
    "crunch",
    "crush",
    "crust",
    "cry",
    "crystal",
    "cub",
    "cube",
    "cuckoo",
    "cucumber",
    "cue",
    "cuisine",
    "culture",
    "culvert",
    "cummerbund",
    "cup",
    "cupboard",
    "cupcake",
    "curd",
    "cure",
    "curio",
    "curiosity",
    "curl",
    "curler",
    "currant",
    "currency",
    "current",
    "curriculum",
    "curry",
    "curse",
    "cursor",
    "curtailment",
    "curtain",
    "curve",
    "cushion",
    "custard",
    "custom",
    "customer",
    "cut",
    "cuticle",
    "cutlet",
    "cutover",
    "cutting",
    "cycle",
    "cyclone",
    "cylinder",
    "cymbal",
    "cynic",
    "daffodil",
    "dahlia",
    "daikon",
    "daily",
    "dairy",
    "daisy",
    "damage",
    "dame",
    "dance",
    "dancer",
    "dancing",
    "dandelion",
    "danger",
    "dare",
    "dark",
    "darkness",
    "darn",
    "dart",
    "dash",
    "dashboard",
    "data",
    "database",
    "date",
    "daughter",
    "dawn",
    "day",
    "daybed",
    "daylight",
    "deal",
    "dealing",
    "dearest",
    "debate",
    "debris",
    "debt",
    "debtor",
    "decade",
    "decadence",
    "decency",
    "decimal",
    "decision",
    "deck",
    "declaration",
    "declination",
    "decline",
    "decoder",
    "decoration",
    "decrease",
    "dedication",
    "deduce",
    "deduction",
    "deed",
    "deep",
    "deer",
    "default",
    "defeat",
    "defendant",
    "defender",
    "defense",
    "deficit",
    "definition",
    "degree",
    "delay",
    "deliberation",
    "delight",
    "delivery",
    "demand",
    "democracy",
    "democrat",
    "demur",
    "den",
    "denim",
    "denominator",
    "density",
    "dentist",
    "department",
    "departure",
    "dependency",
    "dependent",
    "deployment",
    "deposit",
    "deposition",
    "depot",
    "depression",
    "depressive",
    "depth",
    "deputy",
    "derby",
    "derivation",
    "derivative",
    "derrick",
    "descendant",
    "descent",
    "description",
    "desert",
    "design",
    "designation",
    "designer",
    "desire",
    "desk",
    "desktop",
    "dessert",
    "destination",
    "destiny",
    "destruction",
    "detail",
    "detection",
    "detective",
    "detector",
    "detention",
    "detour",
    "developer",
    "developing",
    "development",
    "device",
    "dew",
    "diabetes",
    "diadem",
    "diagnosis",
    "diagram",
    "dial",
    "dialect",
    "dialogue",
    "diam",
    "diamond",
    "diaper",
    "diaphragm",
    "diarist",
    "diary",
    "dibble",
    "dick",
    "diction",
    "dictionary",
    "diesel",
    "diet",
    "difference",
    "difficulty",
    "diffuse",
    "dig",
    "digestion",
    "digestive",
    "digger",
    "digging",
    "digit",
    "dignity",
    "dill",
    "dilution",
    "dime",
    "dimension",
    "dimple",
    "diner",
    "dinghy",
    "dining",
    "dinner",
    "dinosaur",
    "dioxide",
    "dip",
    "diploma",
    "diplomacy",
    "direction",
    "directive",
    "director",
    "directory",
    "dirt",
    "disadvantage",
    "disagreement",
    "disaster",
    "discharge",
    "discipline",
    "disclaimer",
    "disclosure",
    "disco",
    "disconnection",
    "discount",
    "discourse",
    "discovery",
    "discrepancy",
    "discretion",
    "discussion",
    "disdain",
    "disembodiment",
    "disengagement",
    "disguise",
    "dish",
    "dishwasher",
    "disk",
    "disparity",
    "dispatch",
    "display",
    "disposal",
    "disposer",
    "disposition",
    "dispute",
    "disregard",
    "disruption",
    "dissemination",
    "dissonance",
    "distance",
    "distinction",
    "distortion",
    "distribution",
    "distributor",
    "district",
    "divan",
    "diver",
    "diversity",
    "divide",
    "dividend",
    "divider",
    "diving",
    "division",
    "doc",
    "dock",
    "doctor",
    "doctorate",
    "doctrine",
    "document",
    "documentary",
    "doe",
    "dog",
    "dogsled",
    "dogwood",
    "doing",
    "doll",
    "dollar",
    "dollop",
    "dolman",
    "dolor",
    "dolphin",
    "domain",
    "dome",
    "donation",
    "donkey",
    "donor",
    "donut",
    "door",
    "doorbell",
    "doorknob",
    "doorpost",
    "doorway",
    "dory",
    "dose",
    "dot",
    "double",
    "doubling",
    "doubt",
    "doubter",
    "dough",
    "doughnut",
    "down",
    "downfall",
    "downgrade",
    "download",
    "downstairs",
    "downtown",
    "downturn",
    "dozen",
    "draft",
    "drag",
    "dragon",
    "dragonfly",
    "dragonfruit",
    "dragster",
    "drain",
    "drainage",
    "drake",
    "drama",
    "drapes",
    "draw",
    "drawbridge",
    "drawer",
    "drawing",
    "dream",
    "dreamer",
    "dredger",
    "dress",
    "dresser",
    "dressing",
    "drill",
    "drink",
    "drinking",
    "drive",
    "driver",
    "driveway",
    "driving",
    "drizzle",
    "drop",
    "drudgery",
    "drum",
    "drummer",
    "dryer",
    "duck",
    "duckling",
    "dud",
    "dude",
    "due",
    "duel",
    "dueling",
    "duffel",
    "dugout",
    "dune",
    "duration",
    "durian",
    "dusk",
    "dust",
    "duster",
    "duty",
    "dwarf",
    "dwell",
    "dwelling",
    "dynamics",
    "dynamo",
    "dynasty",
    "eagle",
    "eaglet",
    "ear",
    "eardrum",
    "earmuffs",
    "earnings",
    "earplug",
    "earring",
    "earrings",
    "earth",
    "earthquake",
    "earthworm",
    "ease",
    "easel",
    "east",
    "eating",
    "eaves",
    "eavesdropper",
    "eclipse",
    "ecliptic",
    "ecology",
    "economics",
    "economy",
    "ecosystem",
    "eddy",
    "edge",
    "edger",
    "edible",
    "editing",
    "edition",
    "editor",
    "editorial",
    "education",
    "eel",
    "effect",
    "effective",
    "effector",
    "efficacy",
    "efficiency",
    "effort",
    "egg",
    "egghead",
    "eggnog",
    "eggplant",
    "ego",
    "elbow",
    "elderberry",
    "election",
    "electricity",
    "electronics",
    "element",
    "elephant",
    "elevation",
    "elevator",
    "eleventh",
    "elf",
    "elicit",
    "eligibility",
    "elimination",
    "elite",
    "elixir",
    "elk",
    "ellipse",
    "elm",
    "elongation",
    "elver",
    "email",
    "emanate",
    "embassy",
    "embossing",
    "emerald",
    "emergence",
    "emergent",
    "emery",
    "emission",
    "emitter",
    "emotion",
    "emphasis",
    "empire",
    "employ",
    "employee",
    "employer",
    "employment",
    "empowerment",
    "emu",
    "enactment",
    "enclave",
    "enclosure",
    "encounter",
    "encouragement",
    "encyclopedia",
    "end",
    "endive",
    "endorsement",
    "endpoint",
    "enemy",
    "energy",
    "enforcement",
    "engagement",
    "engine",
    "engineer",
    "engineering",
    "enigma",
    "enjoyment",
    "enquiry",
    "enrollment",
    "enterprise",
    "entertainment",
    "enthusiasm",
    "entirety",
    "entity",
    "entrance",
    "entree",
    "entrepreneur",
    "entry",
    "envelope",
    "environment",
    "envy",
    "enzyme",
    "epic",
    "episode",
    "epoch",
    "epoxy",
    "equal",
    "equality",
    "equation",
    "equinox",
    "equipment",
    "equity",
    "equivalent",
    "era",
    "eraser",
    "erosion",
    "error",
    "escalator",
    "escape",
    "escort",
    "essay",
    "essence",
    "essential",
    "establishment",
    "estate",
    "estimate",
    "estuary",
    "eternity",
    "ethernet",
    "ethics",
    "evaluation",
    "evaluator",
    "evaporation",
    "eve",
    "evening",
    "event",
    "everybody",
    "everyone",
    "everything",
    "evidence",
    "evocation",
    "evolution",
    "exaggeration",
    "exam",
    "examination",
    "examiner",
    "example",
    "excellence",
    "exception",
    "excerpt",
    "excess",
    "exchange",
    "exclamation",
    "excursion",
    "excuse",
    "executive",
    "executor",
    "exercise",
    "exhaust",
    "exhaustion",
    "exhibit",
    "exhibition",
    "exile",
    "existence",
    "exit",
    "expansion",
    "expectancy",
    "expectation",
    "expedition",
    "expense",
    "experience",
    "experiment",
    "expert",
    "expertise",
    "explanation",
    "exploration",
    "explorer",
    "export",
    "expose",
    "exposition",
    "exposure",
    "expression",
    "extension",
    "extent",
    "exterior",
    "external",
    "extreme",
    "eye",
    "eyeball",
    "eyebrow",
    "eyebrows",
    "eyeglasses",
    "eyelash",
    "eyelashes",
    "eyelid",
    "eyelids",
    "eyeliner",
    "eyestrain",
    "fabric",
    "face",
    "facet",
    "facility",
    "facsimile",
    "fact",
    "factor",
    "factory",
    "faculty",
    "fail",
    "failure",
    "fairness",
    "fairy",
    "fall",
    "fallacy",
    "fame",
    "familiar",
    "familiarity",
    "family",
    "fan",
    "fang",
    "fanlight",
    "farm",
    "farmer",
    "farming",
    "farmland",
    "farrow",
    "fascia",
    "fashion",
    "fat",
    "fate",
    "father",
    "fatigue",
    "faucet",
    "fault",
    "fava",
    "favor",
    "favorite",
    "fawn",
    "fax",
    "fear",
    "feast",
    "feather",
    "feature",
    "federation",
    "fedora",
    "fee",
    "feed",
    "feedback",
    "feeding",
    "feel",
    "feeling",
    "fellow",
    "fen",
    "fence",
    "fencing",
    "fender",
    "fennel",
    "ferret",
    "ferry",
    "ferryboat",
    "fertilizer",
    "festival",
    "few",
    "fiber",
    "fiberglass",
    "fibre",
    "fiction",
    "fiddle",
    "field",
    "fiery",
    "fiesta",
    "fifth",
    "fig",
    "fight",
    "fighter",
    "figure",
    "figurine",
    "file",
    "filing",
    "fill",
    "fillet",
    "filly",
    "film",
    "filter",
    "final",
    "finance",
    "financing",
    "finding",
    "fine",
    "finer",
    "finger",
    "fingerling",
    "fingernail",
    "finish",
    "finisher",
    "fir",
    "fire",
    "fireman",
    "fireplace",
    "firewall",
    "firm",
    "first",
    "fish",
    "fishbone",
    "fisherman",
    "fishery",
    "fishing",
    "fishnet",
    "fit",
    "fitness",
    "fix",
    "fixture",
    "flag",
    "flair",
    "flame",
    "flan",
    "flanker",
    "flare",
    "flash",
    "flat",
    "flatboat",
    "flavor",
    "flax",
    "fleck",
    "fledgling",
    "fleece",
    "flesh",
    "flexibility",
    "flick",
    "flicker",
    "flight",
    "flint",
    "flock",
    "flood",
    "floodplain",
    "floor",
    "flour",
    "flow",
    "flower",
    "fluke",
    "flume",
    "flung",
    "flute",
    "fly",
    "flytrap",
    "foal",
    "foam",
    "fob",
    "focus",
    "fog",
    "fold",
    "folder",
    "folk",
    "folklore",
    "follower",
    "following",
    "fondue",
    "font",
    "food",
    "foodstuffs",
    "fool",
    "foot",
    "footage",
    "football",
    "footnote",
    "footprint",
    "footrest",
    "footstep",
    "footstool",
    "footwear",
    "forage",
    "forager",
    "foray",
    "force",
    "ford",
    "forearm",
    "forebear",
    "forecast",
    "forehead",
    "foreigner",
    "forelimb",
    "forest",
    "forestry",
    "forever",
    "forgery",
    "fork",
    "form",
    "formal",
    "format",
    "formation",
    "former",
    "formula",
    "fort",
    "forte",
    "fortnight",
    "fortress",
    "fortune",
    "forum",
    "foundation",
    "founder",
    "founding",
    "fountain",
    "fourths",
    "fowl",
    "fox",
    "foxglove",
    "fraction",
    "fragrance",
    "frame",
    "framework",
    "fraud",
    "fraudster",
    "freak",
    "freckle",
    "freedom",
    "freelance",
    "freezer",
    "freezing",
    "freight",
    "freighter",
    "frenzy",
    "freon",
    "frequency",
    "fresco",
    "friction",
    "fridge",
    "friend",
    "friendship",
    "fries",
    "fright",
    "fringe",
    "fritter",
    "frock",
    "frog",
    "front",
    "frontier",
    "frost",
    "frosting",
    "frown",
    "fruit",
    "frustration",
    "fry",
    "fuel",
    "fulfillment",
    "full",
    "fun",
    "function",
    "functionality",
    "fund",
    "funding",
    "fundraising",
    "funeral",
    "fur",
    "furnace",
    "furniture",
    "futon",
    "future",
    "gadget",
    "gaffe",
    "gain",
    "gaiters",
    "gale",
    "gallery",
    "galley",
    "gallon",
    "galoshes",
    "gambling",
    "game",
    "gamebird",
    "gaming",
    "gander",
    "gap",
    "garage",
    "garb",
    "garbage",
    "garden",
    "garlic",
    "garment",
    "garter",
    "gas",
    "gasket",
    "gasoline",
    "gasp",
    "gastronomy",
    "gastropod",
    "gate",
    "gateway",
    "gather",
    "gathering",
    "gator",
    "gauge",
    "gauntlet",
    "gavel",
    "gazebo",
    "gazelle",
    "gear",
    "gearshift",
    "geek",
    "gel",
    "gelatin",
    "gelding",
    "gem",
    "gender",
    "gene",
    "general",
    "generation",
    "generator",
    "generosity",
    "genetics",
    "genie",
    "genius",
    "genre",
    "gentleman",
    "geography",
    "geology",
    "geometry",
    "geranium",
    "gerbil",
    "gesture",
    "geyser",
    "ghost",
    "giant",
    "gift",
    "gig",
    "giggle",
    "ginger",
    "gingerbread",
    "ginseng",
    "giraffe",
    "girdle",
    "girl",
    "girlfriend",
    "glacier",
    "gladiolus",
    "glance",
    "gland",
    "glass",
    "glasses",
    "glee",
    "glen",
    "glider",
    "gliding",
    "glimpse",
    "globe",
    "gloom",
    "glory",
    "glove",
    "glow",
    "glucose",
    "glue",
    "glut",
    "glutamate",
    "gnat",
    "gnu",
    "goal",
    "goat",
    "gobbler",
    "goggles",
    "going",
    "gold",
    "goldfish",
    "golf",
    "gondola",
    "gong",
    "good",
    "goodbye",
    "goodie",
    "goodness",
    "goodnight",
    "goodwill",
    "goose",
    "gopher",
    "gorilla",
    "gosling",
    "gossip",
    "governance",
    "government",
    "governor",
    "gown",
    "grace",
    "grade",
    "gradient",
    "graduate",
    "graduation",
    "graffiti",
    "graft",
    "grain",
    "gram",
    "grammar",
    "gran",
    "grand",
    "grandchild",
    "grandfather",
    "grandma",
    "grandmom",
    "grandmother",
    "grandpa",
    "grandparent",
    "grandson",
    "granny",
    "granola",
    "grant",
    "grape",
    "grapefruit",
    "graph",
    "graphic",
    "grasp",
    "grasshopper",
    "grassland",
    "gratitude",
    "gravel",
    "gravitas",
    "gravity",
    "gravy",
    "gray",
    "grease",
    "greatness",
    "greed",
    "green",
    "greenhouse",
    "greens",
    "grenade",
    "grey",
    "grid",
    "grief",
    "grill",
    "grin",
    "grip",
    "gripper",
    "grit",
    "grocery",
    "ground",
    "group",
    "grouper",
    "grouse",
    "grove",
    "growth",
    "grub",
    "guacamole",
    "guarantee",
    "guard",
    "guava",
    "guerrilla",
    "guess",
    "guest",
    "guestbook",
    "guidance",
    "guide",
    "guideline",
    "guilder",
    "guilt",
    "guilty",
    "guinea",
    "guitar",
    "guitarist",
    "gum",
    "gumshoe",
    "gutter",
    "guy",
    "gym",
    "gymnast",
    "gymnastics",
    "gyro",
    "habit",
    "habitat",
    "hacksaw",
    "hackwork",
    "hail",
    "hair",
    "haircut",
    "hake",
    "half",
    "halibut",
    "hall",
    "halloween",
    "hallway",
    "halt",
    "ham",
    "hamburger",
    "hammer",
    "hammock",
    "hamster",
    "hand",
    "handball",
    "handful",
    "handicap",
    "handle",
    "handlebar",
    "handmaiden",
    "handover",
    "handrail",
    "handsaw",
    "hanger",
    "happening",
    "happiness",
    "harald",
    "harbor",
    "harbour",
    "hardboard",
    "hardcover",
    "hardening",
    "hardhat",
    "hardship",
    "hardware",
    "hare",
    "harm",
    "harmonica",
    "harmonise",
    "harmonize",
    "harmony",
    "harp",
    "harpooner",
    "harvest",
    "harvester",
    "hash",
    "hashtag",
    "hassock",
    "haste",
    "hat",
    "hatbox",
    "hatchet",
    "hatchling",
    "haunt",
    "haven",
    "haversack",
    "havoc",
    "hawk",
    "hay",
    "haze",
    "hazel",
    "hazelnut",
    "head",
    "headache",
    "headlight",
    "headline",
    "headphones",
    "headquarters",
    "headrest",
    "health",
    "hearing",
    "hearsay",
    "heart",
    "heartache",
    "heartbeat",
    "hearth",
    "hearthside",
    "heartwood",
    "heat",
    "heater",
    "heating",
    "heaven",
    "heavy",
    "hectare",
    "hedge",
    "hedgehog",
    "heel",
    "heifer",
    "height",
    "heir",
    "heirloom",
    "helicopter",
    "helium",
    "hellcat",
    "hello",
    "helmet",
    "helo",
    "help",
    "hemisphere",
    "hemp",
    "hen",
    "herb",
    "herbs",
    "heritage",
    "hermit",
    "hero",
    "heron",
    "herring",
    "hesitation",
    "hexagon",
    "heyday",
    "hiccups",
    "hide",
    "hierarchy",
    "high",
    "highland",
    "highlight",
    "highway",
    "hike",
    "hiking",
    "hill",
    "hint",
    "hip",
    "hippodrome",
    "hire",
    "hiring",
    "historian",
    "history",
    "hit",
    "hive",
    "hobbit",
    "hobby",
    "hockey",
    "hoe",
    "hog",
    "hold",
    "holder",
    "holiday",
    "home",
    "homeland",
    "hometown",
    "homework",
    "homonym",
    "honesty",
    "honey",
    "honeybee",
    "honeydew",
    "honor",
    "honoree",
    "hood",
    "hoof",
    "hook",
    "hop",
    "hope",
    "hops",
    "horde",
    "horizon",
    "hormone",
    "horn",
    "hornet",
    "horror",
    "horse",
    "horseradish",
    "horst",
    "hose",
    "hosiery",
    "hospice",
    "hospital",
    "hospitality",
    "host",
    "hostel",
    "hostess",
    "hotdog",
    "hotel",
    "hound",
    "hour",
    "hourglass",
    "house",
    "houseboat",
    "household",
    "housewife",
    "housework",
    "housing",
    "hovel",
    "hovercraft",
    "howard",
    "hub",
    "hubcap",
    "hubris",
    "hug",
    "hugger",
    "hull",
    "human",
    "humanity",
    "humidity",
    "hummus",
    "humor",
    "humour",
    "hundred",
    "hunger",
    "hunt",
    "hunter",
    "hunting",
    "hurdle",
    "hurdler",
    "hurricane",
    "hurry",
    "hurt",
    "husband",
    "hut",
    "hutch",
    "hyacinth",
    "hydrant",
    "hydraulics",
    "hydrocarb",
    "hydrocarbon",
    "hydrofoil",
    "hydrogen",
    "hyena",
    "hygienic",
    "hype",
    "hypothesis",
    "ice",
    "iceberg",
    "icebreaker",
    "icecream",
    "icicle",
    "icing",
    "icon",
    "icy",
    "id",
    "idea",
    "ideal",
    "identification",
    "identity",
    "ideology",
    "idiom",
    "igloo",
    "ignorance",
    "ignorant",
    "illegal",
    "illiteracy",
    "illness",
    "illusion",
    "illustration",
    "image",
    "imagination",
    "imbalance",
    "imitation",
    "immortal",
    "impact",
    "impairment",
    "impala",
    "impediment",
    "implement",
    "implementation",
    "implication",
    "import",
    "importance",
    "impostor",
    "impress",
    "impression",
    "imprisonment",
    "impropriety",
    "improvement",
    "impudence",
    "impulse",
    "inability",
    "inauguration",
    "inbox",
    "incandescence",
    "incarnation",
    "incense",
    "incentive",
    "inch",
    "incidence",
    "incident",
    "incision",
    "inclusion",
    "income",
    "incompetence",
    "inconvenience",
    "increase",
    "incubation",
    "independence",
    "independent",
    "index",
    "indication",
    "indicator",
    "indigence",
    "individual",
    "industry",
    "inequality",
    "inevitable",
    "infancy",
    "infant",
    "infection",
    "infiltration",
    "infinite",
    "infix",
    "inflammation",
    "inflation",
    "influence",
    "influx",
    "info",
    "information",
    "infrastructure",
    "infusion",
    "ingrate",
    "ingredient",
    "inhabitant",
    "inheritance",
    "inhibition",
    "inhibitor",
    "initial",
    "initialise",
    "initialize",
    "initiative",
    "injunction",
    "injury",
    "injustice",
    "ink",
    "inlay",
    "inn",
    "innervation",
    "innocence",
    "innocent",
    "innovation",
    "input",
    "inquiry",
    "inscription",
    "insect",
    "insert",
    "inside",
    "insight",
    "insolence",
    "insomnia",
    "inspection",
    "inspector",
    "inspiration",
    "installation",
    "instance",
    "instant",
    "instinct",
    "institute",
    "institution",
    "instruction",
    "instructor",
    "instrument",
    "instrumentalist",
    "instrumentation",
    "insulation",
    "insurance",
    "insurgence",
    "insurrection",
    "integer",
    "integral",
    "integration",
    "integrity",
    "intellect",
    "intelligence",
    "intensity",
    "intent",
    "intention",
    "intentionality",
    "interaction",
    "interchange",
    "interest",
    "interface",
    "interior",
    "interject",
    "interloper",
    "internet",
    "interpretation",
    "interpreter",
    "interval",
    "intervenor",
    "intervention",
    "interview",
    "interviewer",
    "introduction",
    "intuition",
    "invader",
    "invasion",
    "invention",
    "inventor",
    "inventory",
    "inverse",
    "inversion",
    "investigation",
    "investigator",
    "investment",
    "investor",
    "invitation",
    "invite",
    "invoice",
    "involvement",
    "iridescence",
    "iris",
    "iron",
    "ironclad",
    "irony",
    "irrigation",
    "island",
    "isolation",
    "issue",
    "item",
    "itinerary",
    "ivory",
    "jack",
    "jackal",
    "jacket",
    "jackfruit",
    "jade",
    "jaguar",
    "jail",
    "jailhouse",
    "jalapeno",
    "jam",
    "jar",
    "jasmine",
    "jaw",
    "jazz",
    "jealousy",
    "jeans",
    "jeep",
    "jelly",
    "jellybeans",
    "jellyfish",
    "jet",
    "jewel",
    "jeweller",
    "jewellery",
    "jewelry",
    "jicama",
    "jiffy",
    "job",
    "jockey",
    "joey",
    "jogging",
    "joke",
    "jot",
    "journal",
    "journalism",
    "journalist",
    "journey",
    "joy",
    "judge",
    "judgment",
    "judo",
    "jug",
    "juggernaut",
    "juice",
    "julienne",
    "jumbo",
    "jump",
    "jumper",
    "jumpsuit",
    "jungle",
    "junior",
    "junk",
    "junker",
    "junket",
    "jury",
    "justice",
    "jute",
    "kale",
    "kangaroo",
    "karate",
    "kayak",
    "kazoo",
    "kebab",
    "keep",
    "keeper",
    "kendo",
    "kennel",
    "ketch",
    "ketchup",
    "kettle",
    "kettledrum",
    "key",
    "keyboard",
    "keyboarding",
    "keystone",
    "kick",
    "kid",
    "kilogram",
    "kilometer",
    "kilt",
    "kimono",
    "kind",
    "kindness",
    "king",
    "kingdom",
    "kingfish",
    "kiosk",
    "kiss",
    "kit",
    "kitchen",
    "kite",
    "kitsch",
    "kitten",
    "kitty",
    "kiwi",
    "knee",
    "kneejerk",
    "knickers",
    "knife",
    "knight",
    "knitting",
    "knock",
    "knot",
    "knowledge",
    "knuckle",
    "koala",
    "kohlrabi",
    "kumquat",
    "lab",
    "label",
    "labor",
    "laboratory",
    "laborer",
    "labour",
    "labourer",
    "lace",
    "lack",
    "lad",
    "ladder",
    "ladle",
    "lady",
    "ladybug",
    "lag",
    "lake",
    "lamb",
    "lambkin",
    "lament",
    "lamp",
    "lanai",
    "land",
    "landform",
    "landing",
    "landmine",
    "landscape",
    "lane",
    "language",
    "lantern",
    "lap",
    "lapdog",
    "laptop",
    "larch",
    "lard",
    "larder",
    "lark",
    "larva",
    "lasagna",
    "lashes",
    "last",
    "latency",
    "lathe",
    "latitude",
    "latte",
    "latter",
    "laugh",
    "laughter",
    "laundry",
    "lava",
    "law",
    "lawmaker",
    "lawn",
    "lawsuit",
    "lawyer",
    "lay",
    "layer",
    "layout",
    "lead",
    "leader",
    "leadership",
    "leading",
    "leaf",
    "league",
    "leaker",
    "leap",
    "learning",
    "leash",
    "leather",
    "leave",
    "leaver",
    "lecture",
    "leek",
    "leeway",
    "left",
    "leg",
    "legacy",
    "legal",
    "legend",
    "legging",
    "legislation",
    "legislator",
    "legislature",
    "legitimacy",
    "legume",
    "leisure",
    "lemon",
    "lemonade",
    "lemur",
    "lender",
    "lending",
    "length",
    "lens",
    "lentil",
    "leopard",
    "lesson",
    "letter",
    "lettuce",
    "level",
    "lever",
    "leverage",
    "leveret",
    "liability",
    "liar",
    "liberty",
    "library",
    "licence",
    "license",
    "licensing",
    "licorice",
    "lid",
    "lie",
    "lieu",
    "lieutenant",
    "life",
    "lifestyle",
    "lifetime",
    "lift",
    "ligand",
    "light",
    "lighting",
    "lightning",
    "lightscreen",
    "likelihood",
    "likeness",
    "lilac",
    "lily",
    "limb",
    "lime",
    "limestone",
    "limit",
    "limitation",
    "limo",
    "line",
    "linen",
    "liner",
    "linguist",
    "linguistics",
    "lining",
    "link",
    "linkage",
    "linseed",
    "lion",
    "lip",
    "lipid",
    "lipstick",
    "liquid",
    "liquidity",
    "list",
    "listening",
    "listing",
    "literate",
    "literature",
    "litigation",
    "litmus",
    "litter",
    "littleneck",
    "livestock",
    "living",
    "lizard",
    "llama",
    "load",
    "loading",
    "loaf",
    "loafer",
    "loan",
    "lobby",
    "lobster",
    "local",
    "locality",
    "location",
    "lock",
    "locker",
    "locket",
    "locomotive",
    "locust",
    "lode",
    "loft",
    "log",
    "logic",
    "login",
    "logistics",
    "logo",
    "lollipop",
    "loneliness",
    "longboat",
    "longitude",
    "look",
    "lookout",
    "loop",
    "loophole",
    "loquat",
    "loss",
    "lot",
    "lotion",
    "lottery",
    "lounge",
    "louse",
    "lout",
    "love",
    "lox",
    "loyalty",
    "luck",
    "luggage",
    "lumber",
    "lumberman",
    "lunch",
    "luncheonette",
    "lunchmeat",
    "lunchroom",
    "lung",
    "lunge",
    "lute",
    "luxury",
    "lychee",
    "lycra",
    "lye",
    "lynx",
    "lyre",
    "lyrics",
    "lysine",
    "macadamia",
    "macaroni",
    "macaroon",
    "macaw",
    "machine",
    "machinery",
    "macrame",
    "macro",
    "macrofauna",
    "maelstrom",
    "maestro",
    "magazine",
    "magic",
    "magnet",
    "magnitude",
    "maid",
    "maiden",
    "mail",
    "mailbox",
    "mailer",
    "mailing",
    "mailman",
    "main",
    "mainland",
    "mainstream",
    "maintainer",
    "maintenance",
    "major",
    "majority",
    "makeover",
    "maker",
    "makeup",
    "making",
    "male",
    "malice",
    "mall",
    "mallard",
    "mallet",
    "mama",
    "mambo",
    "mammoth",
    "man",
    "manacle",
    "management",
    "manager",
    "manatee",
    "mandarin",
    "mandate",
    "mandolin",
    "mangle",
    "mango",
    "mangrove",
    "manhunt",
    "maniac",
    "manicure",
    "manifestation",
    "manipulation",
    "manner",
    "manor",
    "mansard",
    "mansion",
    "mantel",
    "mantle",
    "mantua",
    "manufacturer",
    "manufacturing",
    "many",
    "map",
    "maple",
    "mapping",
    "maracas",
    "marathon",
    "marble",
    "march",
    "mare",
    "margarine",
    "margin",
    "mariachi",
    "marimba",
    "marines",
    "marionberry",
    "mark",
    "marker",
    "market",
    "marketer",
    "marketing",
    "marketplace",
    "marksman",
    "markup",
    "marmalade",
    "marriage",
    "marsh",
    "marshland",
    "marshmallow",
    "marten",
    "mascara",
    "mask",
    "masonry",
    "mass",
    "massage",
    "mast",
    "master",
    "masterpiece",
    "mastoid",
    "mat",
    "match",
    "matchmaker",
    "mate",
    "material",
    "maternity",
    "math",
    "mathematics",
    "matrix",
    "matter",
    "mattock",
    "mattress",
    "max",
    "maximum",
    "maybe",
    "mayonnaise",
    "mayor",
    "meadow",
    "meal",
    "mean",
    "meander",
    "meaning",
    "means",
    "meantime",
    "measure",
    "measurement",
    "meat",
    "meatball",
    "meatloaf",
    "mecca",
    "mechanic",
    "mechanism",
    "med",
    "medal",
    "media",
    "median",
    "medication",
    "medicine",
    "medium",
    "meet",
    "meeting",
    "melatonin",
    "melody",
    "melon",
    "member",
    "membership",
    "membrane",
    "meme",
    "memo",
    "memorial",
    "memory",
    "men",
    "mention",
    "mentor",
    "menu",
    "merchandise",
    "merchant",
    "mercury",
    "meridian",
    "meringue",
    "merit",
    "mess",
    "message",
    "messenger",
    "messy",
    "metabolite",
    "metal",
    "metaphor",
    "meteor",
    "meteorology",
    "meter",
    "methane",
    "method",
    "methodology",
    "metric",
    "metro",
    "metronome",
    "mezzanine",
    "microlending",
    "microphone",
    "microwave",
    "midden",
    "middle",
    "middleman",
    "midline",
    "midnight",
    "midwife",
    "might",
    "migrant",
    "migration",
    "mile",
    "mileage",
    "milepost",
    "milestone",
    "military",
    "milk",
    "milkshake",
    "mill",
    "millennium",
    "millet",
    "millimeter",
    "million",
    "millisecond",
    "millstone",
    "mime",
    "mimosa",
    "min",
    "mincemeat",
    "mind",
    "mine",
    "mineral",
    "mineshaft",
    "mini",
    "minibus",
    "minimalism",
    "minimum",
    "mining",
    "minion",
    "minister",
    "mink",
    "minnow",
    "minor",
    "minority",
    "mint",
    "minute",
    "miracle",
    "mirror",
    "misfit",
    "misnomer",
    "misplacement",
    "misreading",
    "miss",
    "missile",
    "mission",
    "mist",
    "mistake",
    "mister",
    "misunderstand",
    "miter",
    "mitten",
    "mix",
    "mixer",
    "mixture",
    "moat",
    "mob",
    "mobile",
    "mobility",
    "moccasins",
    "mocha",
    "mochi",
    "mode",
    "model",
    "modeling",
    "modem",
    "modernist",
    "modernity",
    "modification",
    "molar",
    "molasses",
    "molding",
    "mole",
    "molecule",
    "mom",
    "moment",
    "money",
    "monger",
    "monitor",
    "monitoring",
    "monk",
    "monkey",
    "monocle",
    "monopoly",
    "monsoon",
    "monster",
    "month",
    "monument",
    "mood",
    "moody",
    "moon",
    "moonlight",
    "moonscape",
    "moonshine",
    "moose",
    "mop",
    "morale",
    "morning",
    "morsel",
    "mortal",
    "mortality",
    "mortgage",
    "mortise",
    "mosquito",
    "most",
    "motel",
    "moth",
    "mother",
    "motion",
    "motivation",
    "motive",
    "motor",
    "motorboat",
    "motorcar",
    "motorcycle",
    "mound",
    "mountain",
    "mouse",
    "mouser",
    "mousse",
    "moustache",
    "mouton",
    "movement",
    "mover",
    "movie",
    "mower",
    "mozzarella",
    "mud",
    "muffin",
    "mug",
    "mukluk",
    "mule",
    "multimedia",
    "muscat",
    "muscle",
    "museum",
    "mushroom",
    "music",
    "musician",
    "muskrat",
    "mussel",
    "mustache",
    "mustard",
    "mutation",
    "mutt",
    "mutton",
    "mystery",
    "myth",
    "mythology",
    "nail",
    "name",
    "naming",
    "napkin",
    "narrative",
    "nasal",
    "nation",
    "nationality",
    "native",
    "nature",
    "navigation",
    "necessity",
    "necklace",
    "necktie",
    "nectar",
    "nectarine",
    "need",
    "needle",
    "neglect",
    "negotiation",
    "neighbor",
    "neighborhood",
    "neighbour",
    "neighbourhood",
    "neon",
    "neonate",
    "nephew",
    "nerve",
    "nest",
    "nestling",
    "nestmate",
    "net",
    "netball",
    "netbook",
    "network",
    "networking",
    "neuron",
    "news",
    "newsletter",
    "newspaper",
    "newsprint",
    "newsstand",
    "nexus",
    "nibble",
    "nicety",
    "niche",
    "nick",
    "nickel",
    "nickname",
    "niece",
    "night",
    "nightclub",
    "nightgown",
    "nightingale",
    "nightlife",
    "nightlight",
    "nightmare",
    "ninja",
    "nit",
    "nitrogen",
    "nobody",
    "nod",
    "node",
    "noir",
    "noise",
    "nonconformist",
    "nondisclosure",
    "nonsense",
    "noodle",
    "noodles",
    "noon",
    "norm",
    "normal",
    "normalisation",
    "normalization",
    "north",
    "nose",
    "notation",
    "note",
    "notebook",
    "notepad",
    "nothing",
    "notice",
    "notion",
    "notoriety",
    "nougat",
    "noun",
    "nourishment",
    "novel",
    "nudge",
    "number",
    "numeracy",
    "numeric",
    "nurse",
    "nursery",
    "nursing",
    "nurture",
    "nut",
    "nutmeg",
    "nutrient",
    "nutrition",
    "nylon",
    "nymph",
    "oak",
    "oar",
    "oasis",
    "oat",
    "oatmeal",
    "oats",
    "obedience",
    "obi",
    "object",
    "objection",
    "objective",
    "obligation",
    "oboe",
    "observation",
    "observatory",
    "obsession",
    "obsidian",
    "obstacle",
    "occasion",
    "occupation",
    "occurrence",
    "ocean",
    "ocelot",
    "octagon",
    "octave",
    "octavo",
    "octet",
    "octopus",
    "odometer",
    "odyssey",
    "oeuvre",
    "offence",
    "offense",
    "offer",
    "offering",
    "office",
    "officer",
    "official",
    "offset",
    "oil",
    "okra",
    "oldie",
    "oleo",
    "olive",
    "omega",
    "omelet",
    "omission",
    "omnivore",
    "oncology",
    "onion",
    "online",
    "onset",
    "opening",
    "opera",
    "operating",
    "operation",
    "operator",
    "opinion",
    "opossum",
    "opponent",
    "opportunist",
    "opportunity",
    "opposite",
    "opposition",
    "optimal",
    "optimisation",
    "optimist",
    "optimization",
    "option",
    "orange",
    "orangutan",
    "orator",
    "orchard",
    "orchestra",
    "orchid",
    "order",
    "ordinary",
    "ordination",
    "ore",
    "oregano",
    "organ",
    "organisation",
    "organising",
    "organization",
    "organizing",
    "orient",
    "orientation",
    "origin",
    "original",
    "originality",
    "ornament",
    "osmosis",
    "osprey",
    "ostrich",
    "other",
    "otter",
    "ottoman",
    "ounce",
    "outback",
    "outcome",
    "outfielder",
    "outfit",
    "outhouse",
    "outlaw",
    "outlay",
    "outlet",
    "outline",
    "outlook",
    "output",
    "outrage",
    "outrigger",
    "outrun",
    "outset",
    "outside",
    "oval",
    "oven",
    "overcharge",
    "overclocking",
    "overcoat",
    "overexertion",
    "overflight",
    "overhead",
    "overheard",
    "overload",
    "overnighter",
    "overshoot",
    "oversight",
    "overview",
    "owl",
    "owner",
    "ownership",
    "ox",
    "oxford",
    "oxygen",
    "oyster",
    "ozone",
    "pace",
    "pacemaker",
    "pack",
    "package",
    "packaging",
    "packet",
    "pad",
    "paddle",
    "paddock",
    "page",
    "pail",
    "pain",
    "paint",
    "painter",
    "painting",
    "paintwork",
    "pair",
    "pajamas",
    "palace",
    "palate",
    "palm",
    "pamphlet",
    "pan",
    "pancake",
    "pancreas",
    "panda",
    "panel",
    "panic",
    "pannier",
    "panpipe",
    "pansy",
    "panther",
    "pantry",
    "pants",
    "pantsuit",
    "pantyhose",
    "papa",
    "papaya",
    "paper",
    "paperback",
    "paperwork",
    "parable",
    "parachute",
    "parade",
    "paradise",
    "paragraph",
    "paramedic",
    "parameter",
    "paranoia",
    "parcel",
    "parchment",
    "pard",
    "pardon",
    "parent",
    "parenthesis",
    "parenting",
    "park",
    "parka",
    "parking",
    "parliament",
    "parole",
    "parrot",
    "parser",
    "parsley",
    "parsnip",
    "part",
    "participant",
    "participation",
    "particle",
    "particular",
    "partner",
    "partnership",
    "partridge",
    "party",
    "pass",
    "passage",
    "passbook",
    "passenger",
    "passing",
    "passion",
    "passive",
    "passport",
    "password",
    "past",
    "pasta",
    "paste",
    "pastry",
    "pasture",
    "pat",
    "patch",
    "pate",
    "patent",
    "patentee",
    "path",
    "pathology",
    "pathway",
    "patience",
    "patient",
    "patina",
    "patio",
    "patrimony",
    "patriot",
    "patrol",
    "patroller",
    "patrolling",
    "patron",
    "pattern",
    "patty",
    "pattypan",
    "pause",
    "pavement",
    "pavilion",
    "paw",
    "pawnshop",
    "pay",
    "payee",
    "payment",
    "payoff",
    "pea",
    "peace",
    "peach",
    "peacoat",
    "peacock",
    "peak",
    "peanut",
    "pear",
    "pearl",
    "peasant",
    "pecan",
    "pedal",
    "peek",
    "peen",
    "peer",
    "pegboard",
    "pelican",
    "pelt",
    "pen",
    "penalty",
    "pence",
    "pencil",
    "pendant",
    "pendulum",
    "penguin",
    "penicillin",
    "peninsula",
    "pennant",
    "penny",
    "pension",
    "pentagon",
    "peony",
    "people",
    "pepper",
    "pepperoni",
    "percent",
    "percentage",
    "perception",
    "perch",
    "perennial",
    "perfection",
    "performance",
    "perfume",
    "period",
    "periodical",
    "peripheral",
    "permafrost",
    "permission",
    "permit",
    "persimmon",
    "person",
    "personal",
    "personality",
    "personnel",
    "perspective",
    "pest",
    "pet",
    "petal",
    "petition",
    "petitioner",
    "petticoat",
    "pew",
    "pharmacist",
    "phase",
    "pheasant",
    "phenomenon",
    "phenotype",
    "pheromone",
    "philanthropy",
    "philosopher",
    "philosophy",
    "phone",
    "phosphate",
    "photo",
    "photograph",
    "photographer",
    "photography",
    "phrase",
    "phrasing",
    "physical",
    "physics",
    "physiology",
    "pianist",
    "piano",
    "piccolo",
    "pick",
    "pickax",
    "pickaxe",
    "picket",
    "pickle",
    "pickup",
    "picnic",
    "picture",
    "picturesque",
    "pie",
    "piece",
    "pier",
    "piety",
    "pig",
    "pigeon",
    "piglet",
    "pigpen",
    "pigsty",
    "pike",
    "pilaf",
    "pile",
    "pilgrim",
    "pilgrimage",
    "pill",
    "pillar",
    "pillbox",
    "pillow",
    "pilot",
    "pimple",
    "pin",
    "pinafore",
    "pine",
    "pineapple",
    "pinecone",
    "ping",
    "pink",
    "pinkie",
    "pinot",
    "pinstripe",
    "pint",
    "pinto",
    "pinworm",
    "pioneer",
    "pipe",
    "pipeline",
    "piracy",
    "pirate",
    "pistol",
    "pit",
    "pita",
    "pitch",
    "pitcher",
    "pitching",
    "pith",
    "pizza",
    "place",
    "placebo",
    "placement",
    "placode",
    "plagiarism",
    "plain",
    "plaintiff",
    "plan",
    "plane",
    "planet",
    "planning",
    "plant",
    "plantation",
    "planter",
    "planula",
    "plaster",
    "plasterboard",
    "plastic",
    "plate",
    "platelet",
    "platform",
    "platinum",
    "platter",
    "platypus",
    "play",
    "player",
    "playground",
    "playroom",
    "playwright",
    "plea",
    "pleasure",
    "pleat",
    "pledge",
    "plenty",
    "plier",
    "pliers",
    "plight",
    "plot",
    "plough",
    "plover",
    "plow",
    "plowman",
    "plug",
    "plugin",
    "plum",
    "plumber",
    "plume",
    "plunger",
    "plywood",
    "pneumonia",
    "pocket",
    "pocketbook",
    "pod",
    "podcast",
    "poem",
    "poet",
    "poetry",
    "poignance",
    "point",
    "poison",
    "poisoning",
    "poker",
    "pole",
    "polenta",
    "police",
    "policeman",
    "policy",
    "polish",
    "politician",
    "politics",
    "poll",
    "polliwog",
    "pollutant",
    "pollution",
    "polo",
    "polyester",
    "polyp",
    "pomegranate",
    "pomelo",
    "pompom",
    "poncho",
    "pond",
    "pony",
    "pool",
    "poor",
    "pop",
    "popcorn",
    "poppy",
    "popsicle",
    "popularity",
    "population",
    "populist",
    "porcelain",
    "porch",
    "porcupine",
    "pork",
    "porpoise",
    "port",
    "porter",
    "portfolio",
    "porthole",
    "portion",
    "portrait",
    "position",
    "possession",
    "possibility",
    "possible",
    "post",
    "postage",
    "postbox",
    "poster",
    "postfix",
    "pot",
    "potato",
    "potential",
    "pottery",
    "pouch",
    "poultry",
    "pound",
    "pounding",
    "poverty",
    "powder",
    "power",
    "practice",
    "practitioner",
    "prairie",
    "praise",
    "precedence",
    "precedent",
    "precipitation",
    "precision",
    "predecessor",
    "preface",
    "preference",
    "prefix",
    "prejudice",
    "prelude",
    "premeditation",
    "premier",
    "premise",
    "premium",
    "preoccupation",
    "preparation",
    "prescription",
    "presence",
    "present",
    "presentation",
    "preservation",
    "preserves",
    "presidency",
    "president",
    "press",
    "pressroom",
    "pressure",
    "prestige",
    "presume",
    "pretzel",
    "prevalence",
    "prevention",
    "prey",
    "price",
    "pricing",
    "pride",
    "primary",
    "primate",
    "prince",
    "princess",
    "principal",
    "principle",
    "print",
    "printer",
    "printing",
    "prior",
    "priority",
    "prison",
    "prisoner",
    "privacy",
    "private",
    "privilege",
    "prize",
    "prizefight",
    "probability",
    "probation",
    "probe",
    "problem",
    "procedure",
    "proceedings",
    "process",
    "processing",
    "processor",
    "proctor",
    "procurement",
    "produce",
    "producer",
    "product",
    "production",
    "productivity",
    "profession",
    "professional",
    "professor",
    "profile",
    "profit",
    "progenitor",
    "program",
    "programme",
    "programming",
    "progress",
    "progression",
    "prohibition",
    "project",
    "promenade",
    "promise",
    "promotion",
    "prompt",
    "pronoun",
    "proof",
    "propaganda",
    "propane",
    "property",
    "prophet",
    "proponent",
    "proportion",
    "proposal",
    "proposition",
    "proprietor",
    "prose",
    "prosecution",
    "prosecutor",
    "prospect",
    "prosperity",
    "prostanoid",
    "prostrate",
    "protection",
    "protein",
    "protest",
    "protocol",
    "providence",
    "provider",
    "province",
    "provision",
    "prow",
    "proximal",
    "proximity",
    "prune",
    "pruner",
    "psychology",
    "pub",
    "public",
    "publication",
    "publicity",
    "publisher",
    "publishing",
    "pudding",
    "puddle",
    "puffin",
    "pug",
    "puggle",
    "pulley",
    "pulse",
    "puma",
    "pump",
    "pumpernickel",
    "pumpkin",
    "pumpkinseed",
    "pun",
    "punch",
    "punctuation",
    "punishment",
    "pup",
    "pupa",
    "pupil",
    "puppet",
    "puppy",
    "purchase",
    "purity",
    "purple",
    "purpose",
    "purr",
    "purse",
    "pursuit",
    "push",
    "put",
    "puzzle",
    "pyramid",
    "quadrant",
    "quail",
    "qualification",
    "quality",
    "quantity",
    "quart",
    "quarter",
    "quartet",
    "quartz",
    "queen",
    "query",
    "quest",
    "question",
    "questioner",
    "questionnaire",
    "quiche",
    "quicksand",
    "quiet",
    "quill",
    "quilt",
    "quince",
    "quinoa",
    "quit",
    "quiver",
    "quota",
    "quotation",
    "quote",
    "rabbit",
    "raccoon",
    "race",
    "racer",
    "racing",
    "rack",
    "radar",
    "radiator",
    "radio",
    "radish",
    "raffle",
    "raft",
    "rag",
    "rage",
    "raid",
    "rail",
    "railing",
    "railroad",
    "railway",
    "raiment",
    "rain",
    "rainbow",
    "raincoat",
    "rainmaker",
    "rainstorm",
    "rainy",
    "raise",
    "raisin",
    "rake",
    "rally",
    "ram",
    "rambler",
    "ramen",
    "ramie",
    "ranch",
    "rancher",
    "range",
    "ranger",
    "rank",
    "rap",
    "raspberry",
    "rat",
    "rate",
    "ratepayer",
    "rating",
    "ratio",
    "rationale",
    "rations",
    "raven",
    "ravioli",
    "rawhide",
    "ray",
    "rayon",
    "razor",
    "reach",
    "reactant",
    "reaction",
    "read",
    "reader",
    "readiness",
    "reading",
    "real",
    "reality",
    "realization",
    "realm",
    "reamer",
    "rear",
    "reason",
    "reasoning",
    "rebel",
    "rebellion",
    "reboot",
    "recall",
    "receipt",
    "receiver",
    "reception",
    "receptor",
    "recess",
    "recession",
    "recipe",
    "recipient",
    "reciprocity",
    "reclamation",
    "recliner",
    "recognition",
    "recollection",
    "record",
    "recorder",
    "recording",
    "recovery",
    "recreation",
    "recruit",
    "rectangle",
    "red",
    "redesign",
    "redirect",
    "rediscovery",
    "reduction",
    "reef",
    "refectory",
    "reference",
    "referendum",
    "reflection",
    "reform",
    "refreshments",
    "refrigerator",
    "refuge",
    "refund",
    "refusal",
    "refuse",
    "regard",
    "regime",
    "region",
    "regionalism",
    "register",
    "registration",
    "registry",
    "regret",
    "regulation",
    "regulator",
    "reindeer",
    "reject",
    "relation",
    "relationship",
    "relative",
    "relaxation",
    "relay",
    "release",
    "reliability",
    "relief",
    "religion",
    "relish",
    "reluctance",
    "remains",
    "remark",
    "reminder",
    "remnant",
    "remote",
    "removal",
    "renaissance",
    "rent",
    "reorganisation",
    "reorganization",
    "repair",
    "reparation",
    "repayment",
    "repeat",
    "replacement",
    "replica",
    "replication",
    "reply",
    "report",
    "reporter",
    "reporting",
    "repository",
    "representation",
    "representative",
    "reprocessing",
    "republic",
    "republican",
    "reputation",
    "request",
    "requirement",
    "resale",
    "rescue",
    "research",
    "researcher",
    "resemblance",
    "reservation",
    "reserve",
    "reservoir",
    "reset",
    "residence",
    "resident",
    "residue",
    "resist",
    "resistance",
    "resolution",
    "resolve",
    "resort",
    "resource",
    "respect",
    "respite",
    "response",
    "responsibility",
    "rest",
    "restaurant",
    "restoration",
    "restriction",
    "restroom",
    "restructuring",
    "result",
    "resume",
    "retailer",
    "retention",
    "rethinking",
    "retina",
    "retirement",
    "retouching",
    "retreat",
    "retrospect",
    "retrospective",
    "return",
    "reunion",
    "reveal",
    "revelation",
    "revenant",
    "revenge",
    "revenue",
    "reversal",
    "reverse",
    "review",
    "revival",
    "revolution",
    "reward",
    "rhetoric",
    "rhinoceros",
    "rhubarb",
    "rhyme",
    "rhythm",
    "rib",
    "ribbon",
    "rice",
    "riddle",
    "ride",
    "rider",
    "ridge",
    "riding",
    "rifle",
    "right",
    "rim",
    "ring",
    "riot",
    "rip",
    "ripple",
    "rise",
    "riser",
    "risk",
    "rite",
    "ritual",
    "river",
    "riverbed",
    "rivulet",
    "road",
    "roadway",
    "roar",
    "roast",
    "robe",
    "robin",
    "robot",
    "robotics",
    "rock",
    "rocker",
    "rocket",
    "rod",
    "role",
    "roll",
    "roller",
    "romaine",
    "romance",
    "roof",
    "room",
    "roommate",
    "rooster",
    "root",
    "rope",
    "rose",
    "rosemary",
    "roster",
    "rostrum",
    "rotation",
    "round",
    "roundabout",
    "route",
    "router",
    "routine",
    "row",
    "rowboat",
    "rowing",
    "rubbish",
    "rubric",
    "ruby",
    "ruckus",
    "rudiment",
    "ruffle",
    "rug",
    "rugby",
    "ruin",
    "rule",
    "ruler",
    "ruling",
    "rum",
    "rumor",
    "run",
    "runaway",
    "runner",
    "running",
    "runway",
    "rush",
    "rust",
    "rutabaga",
    "rye",
    "sabre",
    "sac",
    "sack",
    "saddle",
    "sadness",
    "safari",
    "safe",
    "safeguard",
    "safety",
    "saffron",
    "sage",
    "sail",
    "sailboat",
    "sailing",
    "sailor",
    "saint",
    "sake",
    "salad",
    "salami",
    "salary",
    "sale",
    "salesman",
    "salmon",
    "salon",
    "saloon",
    "salsa",
    "salt",
    "salute",
    "sample",
    "samurai",
    "sanction",
    "sanctity",
    "sanctuary",
    "sand",
    "sandal",
    "sandbar",
    "sandpaper",
    "sandwich",
    "sanity",
    "sardine",
    "sari",
    "sarong",
    "sash",
    "satellite",
    "satin",
    "satire",
    "satisfaction",
    "sauce",
    "saucer",
    "sauerkraut",
    "sausage",
    "savage",
    "savannah",
    "saving",
    "savings",
    "savior",
    "saviour",
    "savory",
    "saw",
    "saxophone",
    "scaffold",
    "scale",
    "scallion",
    "scallops",
    "scalp",
    "scam",
    "scanner",
    "scarecrow",
    "scarf",
    "scenario",
    "scene",
    "scenery",
    "scent",
    "schedule",
    "scheduling",
    "schema",
    "scheme",
    "scholar",
    "scholarship",
    "school",
    "schoolhouse",
    "schooner",
    "science",
    "scientist",
    "scimitar",
    "scissors",
    "scooter",
    "scope",
    "score",
    "scorn",
    "scorpion",
    "scotch",
    "scout",
    "scow",
    "scrambled",
    "scrap",
    "scraper",
    "scratch",
    "screamer",
    "screen",
    "screening",
    "screenwriting",
    "screwdriver",
    "scrim",
    "scrip",
    "script",
    "scrutiny",
    "sculpting",
    "sculptural",
    "sculpture",
    "sea",
    "seabass",
    "seafood",
    "seagull",
    "seal",
    "seaplane",
    "search",
    "seashore",
    "seaside",
    "season",
    "seat",
    "seaweed",
    "second",
    "secrecy",
    "secret",
    "secretariat",
    "secretary",
    "section",
    "sectional",
    "sector",
    "security",
    "sediment",
    "seed",
    "seeder",
    "seeker",
    "seep",
    "segment",
    "seizure",
    "selection",
    "self",
    "seller",
    "selling",
    "semantics",
    "semester",
    "semicircle",
    "semicolon",
    "seminar",
    "senate",
    "senator",
    "sender",
    "senior",
    "sense",
    "sensibility",
    "sensitive",
    "sensitivity",
    "sensor",
    "sentence",
    "sentencing",
    "sentiment",
    "sepal",
    "separation",
    "sequel",
    "sequence",
    "serial",
    "series",
    "serum",
    "serval",
    "servant",
    "server",
    "service",
    "servitude",
    "sesame",
    "session",
    "set",
    "setback",
    "setting",
    "settlement",
    "settler",
    "severity",
    "sewer",
    "shack",
    "shackle",
    "shade",
    "shadow",
    "shadowbox",
    "shakedown",
    "shaker",
    "shallot",
    "shallows",
    "shame",
    "shampoo",
    "shanty",
    "shape",
    "share",
    "shareholder",
    "shark",
    "shaw",
    "shawl",
    "shear",
    "shearling",
    "sheath",
    "shed",
    "sheep",
    "sheet",
    "shelf",
    "shell",
    "shelter",
    "sherbet",
    "sherry",
    "shield",
    "shift",
    "shin",
    "shine",
    "shingle",
    "ship",
    "shipper",
    "shipping",
    "shipyard",
    "shirt",
    "shirtdress",
    "shoat",
    "shock",
    "shoe",
    "shoehorn",
    "shoelace",
    "shoemaker",
    "shoes",
    "shoestring",
    "shofar",
    "shoot",
    "shootdown",
    "shop",
    "shopper",
    "shopping",
    "shore",
    "shoreline",
    "short",
    "shortage",
    "shorts",
    "shortwave",
    "shot",
    "shoulder",
    "shout",
    "shovel",
    "show",
    "shower",
    "shred",
    "shrimp",
    "shrine",
    "shutdown",
    "sibling",
    "sick",
    "sickness",
    "side",
    "sideboard",
    "sideburns",
    "sidecar",
    "sidestream",
    "sidewalk",
    "siding",
    "siege",
    "sigh",
    "sight",
    "sightseeing",
    "sign",
    "signal",
    "signature",
    "signet",
    "significance",
    "signify",
    "signup",
    "silence",
    "silica",
    "silicon",
    "silk",
    "silkworm",
    "sill",
    "silly",
    "silo",
    "silver",
    "similarity",
    "simple",
    "simplicity",
    "singer",
    "singing",
    "singular",
    "sink",
    "sinuosity",
    "sip",
    "sir",
    "sister",
    "sitar",
    "site",
    "situation",
    "size",
    "skate",
    "skating",
    "skean",
    "skeleton",
    "ski",
    "skiing",
    "skill",
    "skin",
    "skirt",
    "skull",
    "skullcap",
    "skunk",
    "sky",
    "skylight",
    "skyline",
    "skyscraper",
    "skywalk",
    "slang",
    "slapstick",
    "slash",
    "slate",
    "slaw",
    "sled",
    "sledge",
    "sleep",
    "sleepiness",
    "sleeping",
    "sleet",
    "sleuth",
    "slice",
    "slide",
    "slider",
    "slime",
    "slip",
    "slipper",
    "slippers",
    "slope",
    "slot",
    "sloth",
    "slump",
    "smell",
    "smelting",
    "smile",
    "smith",
    "smock",
    "smog",
    "smoke",
    "smoking",
    "smolt",
    "smuggling",
    "snack",
    "snail",
    "snake",
    "snakebite",
    "snap",
    "snarl",
    "sneaker",
    "sneakers",
    "sneeze",
    "sniffle",
    "snob",
    "snorer",
    "snow",
    "snowboarding",
    "snowflake",
    "snowman",
    "snowmobiling",
    "snowplow",
    "snowstorm",
    "snowsuit",
    "snuck",
    "snug",
    "snuggle",
    "soap",
    "soccer",
    "socialism",
    "socialist",
    "society",
    "sociology",
    "sock",
    "socks",
    "soda",
    "sofa",
    "softball",
    "softdrink",
    "softening",
    "software",
    "soil",
    "soldier",
    "sole",
    "solicitation",
    "solicitor",
    "solidarity",
    "solidity",
    "soliloquy",
    "solitaire",
    "solution",
    "solvency",
    "sombrero",
    "somebody",
    "someone",
    "someplace",
    "somersault",
    "something",
    "somewhere",
    "son",
    "sonar",
    "sonata",
    "song",
    "songbird",
    "sonnet",
    "soot",
    "sophomore",
    "soprano",
    "sorbet",
    "sorghum",
    "sorrel",
    "sorrow",
    "sort",
    "soul",
    "soulmate",
    "sound",
    "soundness",
    "soup",
    "source",
    "sourwood",
    "sousaphone",
    "south",
    "southeast",
    "souvenir",
    "sow",
    "soy",
    "soybean",
    "space",
    "spacing",
    "spade",
    "spaghetti",
    "span",
    "spandex",
    "sparerib",
    "spark",
    "sparrow",
    "spasm",
    "spat",
    "spatula",
    "spawn",
    "speaker",
    "speakerphone",
    "speaking",
    "spear",
    "spec",
    "special",
    "specialist",
    "specialty",
    "species",
    "specification",
    "spectacle",
    "spectacles",
    "spectrum",
    "speculation",
    "speech",
    "speed",
    "speedboat",
    "spell",
    "spelling",
    "spelt",
    "spending",
    "sphere",
    "sphynx",
    "spice",
    "spider",
    "spiderling",
    "spike",
    "spill",
    "spinach",
    "spine",
    "spiral",
    "spirit",
    "spit",
    "spite",
    "spleen",
    "splendor",
    "split",
    "spokesman",
    "spokeswoman",
    "sponge",
    "sponsor",
    "sponsorship",
    "spool",
    "spoon",
    "spork",
    "sport",
    "sportsman",
    "spot",
    "spotlight",
    "spouse",
    "sprag",
    "sprat",
    "spray",
    "spread",
    "spreadsheet",
    "spree",
    "spring",
    "sprinkles",
    "sprinter",
    "sprout",
    "spruce",
    "spud",
    "spume",
    "spur",
    "spy",
    "spyglass",
    "square",
    "squash",
    "squatter",
    "squeegee",
    "squid",
    "squirrel",
    "stab",
    "stability",
    "stable",
    "stack",
    "stacking",
    "stadium",
    "staff",
    "stag",
    "stage",
    "stain",
    "stair",
    "staircase",
    "stake",
    "stalk",
    "stall",
    "stallion",
    "stamen",
    "stamina",
    "stamp",
    "stance",
    "stand",
    "standard",
    "standing",
    "standoff",
    "standpoint",
    "star",
    "starboard",
    "start",
    "starter",
    "state",
    "statement",
    "statin",
    "station",
    "statistic",
    "statistics",
    "statue",
    "status",
    "statute",
    "stay",
    "steak",
    "stealth",
    "steam",
    "steamroller",
    "steel",
    "steeple",
    "stem",
    "stench",
    "stencil",
    "step",
    "stepdaughter",
    "stepmother",
    "stepson",
    "stereo",
    "stew",
    "steward",
    "stick",
    "sticker",
    "stiletto",
    "still",
    "stimulation",
    "stimulus",
    "sting",
    "stinger",
    "stitch",
    "stitcher",
    "stock",
    "stockings",
    "stole",
    "stomach",
    "stone",
    "stonework",
    "stool",
    "stop",
    "stopsign",
    "stopwatch",
    "storage",
    "store",
    "storey",
    "storm",
    "story",
    "storyboard",
    "stot",
    "stove",
    "strait",
    "strand",
    "stranger",
    "strap",
    "strategy",
    "straw",
    "strawberry",
    "strawman",
    "stream",
    "street",
    "streetcar",
    "strength",
    "stress",
    "stretch",
    "strife",
    "strike",
    "string",
    "strip",
    "stripe",
    "strobe",
    "stroke",
    "structure",
    "strudel",
    "struggle",
    "stucco",
    "student",
    "studio",
    "study",
    "stuff",
    "stumbling",
    "stump",
    "stupidity",
    "sturgeon",
    "sty",
    "style",
    "styling",
    "stylus",
    "sub",
    "subcomponent",
    "subconscious",
    "subcontractor",
    "subgroup",
    "subject",
    "submarine",
    "submitter",
    "subprime",
    "subroutine",
    "subscription",
    "subsection",
    "subset",
    "subsidence",
    "subsidiary",
    "subsidy",
    "substance",
    "substitution",
    "subtitle",
    "suburb",
    "subway",
    "success",
    "succotash",
    "suede",
    "suet",
    "sugar",
    "suggestion",
    "suit",
    "suitcase",
    "suite",
    "sulfur",
    "sultan",
    "sum",
    "summary",
    "summer",
    "summit",
    "sun",
    "sunbeam",
    "sunbonnet",
    "sundae",
    "sunday",
    "sundial",
    "sunflower",
    "sunglasses",
    "sunlamp",
    "sunlight",
    "sunrise",
    "sunroom",
    "sunset",
    "sunshine",
    "superiority",
    "supermarket",
    "supernatural",
    "supervision",
    "supervisor",
    "supper",
    "supplement",
    "supplier",
    "supply",
    "support",
    "supporter",
    "suppression",
    "supreme",
    "surface",
    "surfboard",
    "surge",
    "surgeon",
    "surgery",
    "surname",
    "surplus",
    "surprise",
    "surround",
    "surroundings",
    "surrounds",
    "survey",
    "survival",
    "survivor",
    "sushi",
    "suspect",
    "suspenders",
    "suspension",
    "sustainment",
    "sustenance",
    "swallow",
    "swamp",
    "swan",
    "swanling",
    "swath",
    "sweat",
    "sweater",
    "sweatshirt",
    "sweatshop",
    "sweatsuit",
    "sweets",
    "swell",
    "swim",
    "swimming",
    "swimsuit",
    "swine",
    "swing",
    "switch",
    "switchboard",
    "switching",
    "swivel",
    "sword",
    "swordfight",
    "swordfish",
    "sycamore",
    "symbol",
    "symmetry",
    "sympathy",
    "symptom",
    "syndicate",
    "syndrome",
    "synergy",
    "synonym",
    "synthesis",
    "syrup",
    "system",
    "tab",
    "tabby",
    "tabernacle",
    "table",
    "tablecloth",
    "tablet",
    "tabletop",
    "tachometer",
    "tackle",
    "taco",
    "tactics",
    "tactile",
    "tadpole",
    "tag",
    "tail",
    "tailbud",
    "tailor",
    "tailspin",
    "takeover",
    "tale",
    "talent",
    "talk",
    "talking",
    "tamale",
    "tambour",
    "tambourine",
    "tan",
    "tandem",
    "tangerine",
    "tank",
    "tanker",
    "tankful",
    "tap",
    "tape",
    "tapioca",
    "target",
    "taro",
    "tarragon",
    "tart",
    "task",
    "tassel",
    "taste",
    "tatami",
    "tattler",
    "tattoo",
    "tavern",
    "tax",
    "taxi",
    "taxicab",
    "taxpayer",
    "tea",
    "teacher",
    "teaching",
    "team",
    "teammate",
    "teapot",
    "tear",
    "tech",
    "technician",
    "technique",
    "technology",
    "tectonics",
    "teen",
    "teenager",
    "teepee",
    "telephone",
    "telescreen",
    "teletype",
    "television",
    "tell",
    "teller",
    "temp",
    "temper",
    "temperature",
    "tempo",
    "temporary",
    "temptation",
    "temptress",
    "tenant",
    "tendency",
    "tender",
    "tenement",
    "tenet",
    "tennis",
    "tenor",
    "tension",
    "tensor",
    "tent",
    "tentacle",
    "tenth",
    "tepee",
    "teriyaki",
    "term",
    "terminal",
    "termination",
    "terminology",
    "termite",
    "terrace",
    "terracotta",
    "terrapin",
    "terrarium",
    "territory",
    "test",
    "testament",
    "testimonial",
    "testimony",
    "testing",
    "text",
    "textbook",
    "textual",
    "texture",
    "thanks",
    "thaw",
    "theater",
    "theft",
    "theme",
    "theory",
    "therapist",
    "therapy",
    "thermals",
    "thermometer",
    "thermostat",
    "thesis",
    "thickness",
    "thief",
    "thigh",
    "thing",
    "thinking",
    "thirst",
    "thistle",
    "thorn",
    "thought",
    "thousand",
    "thread",
    "threat",
    "threshold",
    "thrift",
    "thrill",
    "throat",
    "throne",
    "thrust",
    "thug",
    "thumb",
    "thump",
    "thunder",
    "thunderbolt",
    "thunderhead",
    "thunderstorm",
    "thyme",
    "tiara",
    "tic",
    "tick",
    "ticket",
    "tide",
    "tie",
    "tiger",
    "tights",
    "tile",
    "till",
    "tilt",
    "timbale",
    "timber",
    "time",
    "timeline",
    "timeout",
    "timer",
    "timetable",
    "timing",
    "timpani",
    "tin",
    "tinderbox",
    "tinkle",
    "tintype",
    "tip",
    "tire",
    "tissue",
    "titanium",
    "title",
    "toad",
    "toast",
    "toaster",
    "today",
    "toe",
    "toffee",
    "tofu",
    "tog",
    "toga",
    "toilet",
    "tolerance",
    "tolerant",
    "toll",
    "tomatillo",
    "tomato",
    "tomb",
    "tomography",
    "tomorrow",
    "ton",
    "tonality",
    "tone",
    "tongue",
    "tonic",
    "tonight",
    "tool",
    "toot",
    "tooth",
    "toothbrush",
    "toothpaste",
    "toothpick",
    "top",
    "topic",
    "topsail",
    "toque",
    "toreador",
    "tornado",
    "torso",
    "torte",
    "tortellini",
    "tortilla",
    "tortoise",
    "total",
    "tote",
    "touch",
    "tour",
    "tourism",
    "tourist",
    "tournament",
    "towel",
    "tower",
    "town",
    "townhouse",
    "township",
    "toy",
    "trace",
    "track",
    "tracking",
    "tracksuit",
    "tract",
    "tractor",
    "trade",
    "trader",
    "trading",
    "tradition",
    "traffic",
    "trafficker",
    "tragedy",
    "trail",
    "trailer",
    "trailpatrol",
    "train",
    "trainer",
    "training",
    "trait",
    "tram",
    "trance",
    "transaction",
    "transcript",
    "transfer",
    "transit",
    "transition",
    "translation",
    "transmission",
    "transom",
    "transparency",
    "transport",
    "trap",
    "trapdoor",
    "trash",
    "travel",
    "traveler",
    "tray",
    "treasure",
    "treasury",
    "treat",
    "treatment",
    "treaty",
    "tree",
    "trek",
    "trellis",
    "tremor",
    "trench",
    "trend",
    "triad",
    "trial",
    "triangle",
    "tribe",
    "tributary",
    "trick",
    "trigger",
    "trigonometry",
    "trillion",
    "trim",
    "trinket",
    "trip",
    "tripod",
    "tritone",
    "triumph",
    "trolley",
    "trombone",
    "troop",
    "trooper",
    "trophy",
    "trouble",
    "trousers",
    "trout",
    "trove",
    "trowel",
    "truck",
    "trumpet",
    "trunk",
    "trust",
    "trustee",
    "truth",
    "try",
    "tsunami",
    "tub",
    "tuba",
    "tube",
    "tuber",
    "tug",
    "tugboat",
    "tuition",
    "tulip",
    "tumbler",
    "tummy",
    "tuna",
    "tune",
    "tunic",
    "tunnel",
    "turban",
    "turf",
    "turkey",
    "turmeric",
    "turn",
    "turning",
    "turnip",
    "turnover",
    "turnstile",
    "turret",
    "turtle",
    "tusk",
    "tussle",
    "tutu",
    "tuxedo",
    "tweet",
    "tweezers",
    "twig",
    "twilight",
    "twine",
    "twins",
    "twist",
    "twister",
    "twitter",
    "type",
    "typeface",
    "typewriter",
    "typhoon",
    "ukulele",
    "ultimatum",
    "umbrella",
    "unblinking",
    "uncertainty",
    "uncle",
    "underestimate",
    "underground",
    "underneath",
    "underpass",
    "understanding",
    "understatement",
    "undertaker",
    "underwear",
    "underweight",
    "underwire",
    "underwriting",
    "unemployment",
    "unibody",
    "uniform",
    "uniformity",
    "union",
    "unique",
    "unit",
    "unity",
    "universe",
    "university",
    "update",
    "upgrade",
    "uplift",
    "upper",
    "upstairs",
    "upward",
    "urge",
    "urgency",
    "urn",
    "usage",
    "use",
    "user",
    "usher",
    "usual",
    "utensil",
    "utilisation",
    "utility",
    "utilization",
    "vacation",
    "vacuum",
    "vagrant",
    "valance",
    "valentine",
    "validate",
    "validity",
    "valley",
    "valuable",
    "value",
    "vampire",
    "van",
    "vane",
    "vanilla",
    "vanity",
    "variability",
    "variable",
    "variant",
    "variation",
    "variety",
    "vascular",
    "vase",
    "vault",
    "vaulting",
    "veal",
    "vector",
    "vegetable",
    "vegetarian",
    "vegetation",
    "vehicle",
    "veil",
    "vein",
    "veldt",
    "vellum",
    "velocity",
    "velodrome",
    "velvet",
    "vendor",
    "veneer",
    "vengeance",
    "venison",
    "venom",
    "venti",
    "venture",
    "venue",
    "veranda",
    "verb",
    "verdict",
    "verification",
    "vermicelli",
    "vernacular",
    "verse",
    "version",
    "vertigo",
    "verve",
    "vessel",
    "vest",
    "vestment",
    "vet",
    "veteran",
    "veterinarian",
    "veto",
    "viability",
    "vibe",
    "vibraphone",
    "vibration",
    "vibrissae",
    "vice",
    "vicinity",
    "victim",
    "victory",
    "video",
    "view",
    "viewer",
    "vignette",
    "villa",
    "village",
    "vine",
    "vinegar",
    "vineyard",
    "vintage",
    "vintner",
    "vinyl",
    "viola",
    "violet",
    "violin",
    "virtue",
    "virus",
    "visa",
    "viscose",
    "vise",
    "vision",
    "visit",
    "visitor",
    "visor",
    "vista",
    "visual",
    "vitality",
    "vitamin",
    "vitro",
    "vivo",
    "vixen",
    "vodka",
    "vogue",
    "voice",
    "void",
    "vol",
    "volatility",
    "volcano",
    "volleyball",
    "volume",
    "volunteer",
    "vote",
    "voter",
    "voting",
    "voyage",
    "vulture",
    "wad",
    "wafer",
    "waffle",
    "wage",
    "wagon",
    "waist",
    "waistband",
    "wait",
    "waiter",
    "waiting",
    "waitress",
    "waiver",
    "wake",
    "walk",
    "walker",
    "walking",
    "walkway",
    "wall",
    "wallaby",
    "wallet",
    "walnut",
    "walrus",
    "wannabe",
    "want",
    "war",
    "warden",
    "wardrobe",
    "warlock",
    "warlord",
    "warming",
    "warmth",
    "warning",
    "warrant",
    "warren",
    "warrior",
    "wasabi",
    "wash",
    "washbasin",
    "washcloth",
    "washer",
    "washtub",
    "wasp",
    "waste",
    "wastebasket",
    "wasting",
    "watch",
    "watcher",
    "watchmaker",
    "water",
    "watercress",
    "waterfall",
    "waterfront",
    "watermelon",
    "waterskiing",
    "waterspout",
    "waterwheel",
    "wave",
    "waveform",
    "wax",
    "way",
    "weakness",
    "wealth",
    "weapon",
    "wear",
    "weasel",
    "weather",
    "web",
    "webinar",
    "webmail",
    "webpage",
    "website",
    "wedding",
    "wedge",
    "weeder",
    "weedkiller",
    "week",
    "weekend",
    "weekender",
    "weight",
    "weird",
    "welcome",
    "welfare",
    "well",
    "west",
    "western",
    "wetland",
    "wetsuit",
    "whack",
    "whale",
    "wharf",
    "wheat",
    "wheel",
    "whey",
    "whip",
    "whirlpool",
    "whirlwind",
    "whisker",
    "whiskey",
    "whisper",
    "whistle",
    "white",
    "whole",
    "wholesale",
    "wholesaler",
    "whorl",
    "wick",
    "widget",
    "widow",
    "width",
    "wife",
    "wifi",
    "wild",
    "wildebeest",
    "wilderness",
    "wildlife",
    "will",
    "willow",
    "win",
    "wind",
    "windage",
    "window",
    "windscreen",
    "windshield",
    "wine",
    "winery",
    "wing",
    "wingman",
    "wingtip",
    "wink",
    "winner",
    "winter",
    "wire",
    "wiretap",
    "wiring",
    "wisdom",
    "wiseguy",
    "wish",
    "wisteria",
    "wit",
    "witch",
    "withdrawal",
    "witness",
    "wok",
    "wolf",
    "woman",
    "wombat",
    "wonder",
    "wont",
    "wood",
    "woodchuck",
    "woodland",
    "woodshed",
    "woodwind",
    "wool",
    "woolens",
    "word",
    "wording",
    "work",
    "workbench",
    "worker",
    "workforce",
    "workhorse",
    "working",
    "workout",
    "workplace",
    "workshop",
    "world",
    "worm",
    "worry",
    "worth",
    "wound",
    "wrap",
    "wraparound",
    "wrapper",
    "wrapping",
    "wreck",
    "wrecker",
    "wren",
    "wrench",
    "wrestler",
    "wriggler",
    "wrinkle",
    "wrist",
    "writer",
    "writing",
    "wrong",
    "xylophone",
    "yacht",
    "yahoo",
    "yak",
    "yam",
    "yang",
    "yard",
    "yarn",
    "yawl",
    "year",
    "yeast",
    "yellow",
    "yesterday",
    "yew",
    "yin",
    "yoga",
    "yogurt",
    "yoke",
    "yolk",
    "young",
    "youngster",
    "yourself",
    "youth",
    "yoyo",
    "yurt",
    "zebra",
    "zebrafish",
    "zen",
    "zephyr",
    "zero",
    "ziggurat",
    "zinc",
    "zipper",
    "zither",
    "zombie",
    "zone",
    "zoo",
    "zoologist",
    "zoology",
    "zucchini",
]



from datetime import datetime
from os import mkdir, path
from re import sub
from typing import Callable, Optional, TypeVar

T = TypeVar("T")
U = TypeVar("U")


def get_optional(optional: Optional[T]) -> T:
    """
    General-purpose unwrapping function to handle `Optional`.
    Raises an exception if it's actually `None`, otherwise
    returns the internal type.
    """
    assert optional is not None, "Unwrap called on None"
    return optional


def match_optional(
    optional: Optional[T], none_func: Callable[[], U], some_func: Callable[[T], U]
) -> U:
    """
    General-purpose pattern-matching function to handle `Optional`.
    If it's actually `None`, the `none_func` lambda is called.
    Otherwise, the `some_func` lambda is called with the value.
    """
    if optional is None:
        return none_func()
    else:
        return some_func(optional)


def get_or_else_optional(optional: Optional[T], alt_value: T) -> T:
    """
    General-purpose getter for `Optional`. If it's `None`, returns the `alt_value`.
    Otherwise, returns the contents of `optional`.
    """
    if optional is None:
        return alt_value
    else:
        return optional


def get_or_else_optional_func(optional: Optional[T], func: Callable[[], T]) -> T:
    """
    General-purpose getter for `Optional`. If it's `None`, calls the lambda `func`
    and returns its value. Otherwise, returns the contents of `optional`.
    """
    if optional is None:
        return func()
    else:
        return optional


def flatmap_optional(optional: Optional[T], mapper: Callable[[T], U]) -> Optional[U]:
    """
    General-purpose flatmapping on `Optional`. If it's `None`, returns `None` as well,
    otherwise returns the lambda applied to the contents.
    """
    if optional is None:
        return None
    else:
        return mapper(optional)


def to_ticks(date_time: datetime) -> int:
    """
    Return the number of ticks for a date time
    :param date_time: Date time to convert
    :return: number of ticks
    """
    t0 = datetime(1, 1, 1)
    seconds = int((date_time - t0).total_seconds())
    ticks = seconds * 10 ** 7
    return ticks


def space_between_capitals(base: str) -> str:
    """
    Return a modified string with spaces between capital letters
    :param base: base string
    :return: modified string
    """
    return sub(r"(\w)([A-Z])", r"\1 \2", base)


def make_directory(directory_path: str) -> None:
    """Create a directory only if it does not exist"""
    if not path.exists(directory_path):
        mkdir(directory_path)



from functools import reduce
from random import Random
from typing import TypeVar, Callable, List, Optional, Tuple

from hypothesis.provisional import urls
from hypothesis.strategies import (
    composite,
    emails,
    integers,
    lists,
    SearchStrategy,
    text,
    uuids,
    datetimes,
    one_of,
    just,
)

from electionguard.ballot import PlaintextBallotContest, PlaintextBallot
from electionguard.election import (
    Candidate,
    ElectionType,
    ReportingUnitType,
    VoteVariationType,
    ContactInformation,
    GeopoliticalUnit,
    BallotStyle,
    Language,
    InternationalizedText,
    AnnotatedString,
    Party,
    CandidateContestDescription,
    ReferendumContestDescription,
    ElectionDescription,
    InternalElectionDescription,
    CiphertextElectionContext,
    SelectionDescription,
    ContestDescription,
    make_ciphertext_election_context,
)
from electionguard.encrypt import selection_from
from electionguard.group import ElementModQ
from electionguardtest.elgamal import elgamal_keypairs

_T = TypeVar("_T")
_DrawType = Callable[[SearchStrategy[_T]], _T]

_first_names = [
    "James",
    "Mary",
    "John",
    "Patricia",
    "Robert",
    "Jennifer",
    "Michael",
    "Linda",
    "William",
    "Elizabeth",
    "David",
    "Barbara",
    "Richard",
    "Susan",
    "Joseph",
    "Jessica",
    "Thomas",
    "Sarah",
    "Charles",
    "Karen",
    "Christopher",
    "Nancy",
    "Daniel",
    "Margaret",
    "Matthew",
    "Lisa",
    "Anthony",
    "Betty",
    "Donald",
    "Dorothy",
    "Sylvia",
    "Viktor",
    "Camille",
    "Mirai",
    "Anant",
    "Rohan",
    "François",
    "Altuğ",
    "Sigurður",
    "Böðmóður",
    "Quang Dũng",
]

_last_names = [
    "SMITH",
    "JOHNSON",
    "WILLIAMS",
    "JONES",
    "BROWN",
    "DAVIS",
    "MILLER",
    "WILSON",
    "MOORE",
    "TAYLOR",
    "ANDERSON",
    "THOMAS",
    "JACKSON",
    "WHITE",
    "HARRIS",
    "MARTIN",
    "THOMPSON",
    "GARCIA",
    "MARTINEZ",
    "ROBINSON",
    "CLARK",
    "RODRIGUEZ",
    "LEWIS",
    "LEE",
    "WALKER",
    "HALL",
    "ALLEN",
    "YOUNG",
    "HERNANDEZ",
    "KING",
    "WRIGHT",
    "LOPEZ",
    "HILL",
    "SCOTT",
    "GREEN",
    "ADAMS",
    "BAKER",
    "GONZALEZ",
    "STEELE-LOY",
    "O'CONNOR",
    "ANAND",
    "PATEL",
    "GUPTA",
    "ĐẶNG",
]


@composite
def human_names(draw: _DrawType):
    """
    Generates a string with a human first and last name.
    :param draw: Hidden argument, used by Hypothesis.
    """
    return f"{_first_names[draw(integers(0, len(_first_names) - 1))]} {_last_names[draw(integers(0, len(_last_names) - 1))]}"


@composite
def election_types(draw: _DrawType):
    """
    Generates an `ElectionType`.
    :param draw: Hidden argument, used by Hypothesis.
    """
    n = draw(integers(0, 7))
    return ElectionType(n)


@composite
def reporting_unit_types(draw: _DrawType):
    """
    Generates a `ReportingUnitType` object.
    :param draw: Hidden argument, used by Hypothesis.
    """
    n = draw(integers(0, 28))
    return ReportingUnitType(n)


@composite
def contact_infos(draw: _DrawType):
    """
    Generates a `ContactInformation` object.
    :param draw: Hidden argument, used by Hypothesis.
    """
    # empty lists for email and phone, for now
    return ContactInformation(None, draw(emails()), None, draw(human_names()))


@composite
def two_letter_codes(draw: _DrawType, min_size=2, max_size=2):
    """
    Generates a string with only a few characters, by default 2 letters
    from `a` to `z`, but configurable with the `min_size` and `max_size`
    parameters. Useful when you want something like a two-letter country
    or language code.
    :param draw: Hidden argument, used by Hypothesis.
    :param min_size: minimum number of characters to generate (default: 2)
    :param max_size: maximum number of characters to generate (default: 2)
    """
    return draw(
        text(
            alphabet="abcdefghijklmnopqrstuvwxyz", min_size=min_size, max_size=max_size
        )
    )


@composite
def languages(draw: _DrawType):
    """
    Generates a `Language` object with an arbitrary two-letter string as the code and
    something messier for the text ostensibly written in that language.
    :param draw: Hidden argument, used by Hypothesis.
    """
    return Language(draw(emails()), draw(two_letter_codes()))


@composite
def language_human_names(draw: _DrawType):
    """
    Generates a `Language` object with an arbitrary two-letter string as the code and
    a human name for the text ostensibly written in that language.
    :param draw: Hidden argument, used by Hypothesis.
    """
    return Language(draw(human_names()), draw(two_letter_codes()))


@composite
def internationalized_texts(draw: _DrawType):
    """
    Generates an `InternationalizedText` object with a list of `Language` objects
    within (representing a multilingual string).
    :param draw: Hidden argument, used by Hypothesis.
    """
    return InternationalizedText(draw(lists(languages(), min_size=1, max_size=3)))


@composite
def internationalized_human_names(draw: _DrawType):
    """
    Generates an `InternationalizedText` object with a list of `Language` objects
    within (representing a multilingual human name).
    :param draw: Hidden argument, used by Hypothesis.
    """
    return InternationalizedText(
        draw(lists(language_human_names(), min_size=1, max_size=3))
    )


@composite
def annotated_strings(draw: _DrawType):
    """
    Generates an `AnnotatedString` object with one `Language` and an associated
    `value` string.
    :param draw: Hidden argument, used by Hypothesis.
    """
    s = draw(languages())
    # We're just reusing the "value" string already associated with the language for now.
    return AnnotatedString(annotation=s.language, value=s.value)


@composite
def ballot_styles(
    draw: _DrawType, party_ids: List[Party], geo_units: List[GeopoliticalUnit]
):
    """
    Generates a `BallotStyle` object, which rolls up a list of parties and
    geopolitical units (passed as arguments), with some additional information
    added on as well.
    :param draw: Hidden argument, used by Hypothesis.
    :param party_ids: a list of `Party` objects to be used in this ballot style
    :param geo_units: a list of `GeopoliticalUnit` objects to be used in this ballot style
    """
    assert len(party_ids) > 0
    assert len(geo_units) > 0

    gp_unit_ids = [x.object_id for x in geo_units]
    if len(gp_unit_ids) == 0:
        gp_unit_ids = None

    party_ids = [x.get_party_id() for x in party_ids]
    if len(party_ids) == 0:
        party_ids = None

    image_uri = draw(urls())
    return BallotStyle(str(draw(uuids())), gp_unit_ids, party_ids, image_uri)


@composite
def party_lists(draw: _DrawType, num_parties: int):
    """
    Generates a `List[Party]` of the requested length.
    :param draw: Hidden argument, used by Hypothesis.
    :param num_parties: Number of parties to generate in the list.
    """
    party_names = [f"Party{n}" for n in range(num_parties)]
    party_abbrvs = [f"P{n}" for n in range(num_parties)]

    assert num_parties > 0

    return [
        Party(
            object_id=str(draw(uuids())),
            ballot_name=InternationalizedText([Language(party_names[i], "en")]),
            abbreviation=party_abbrvs[i],
            color=None,
            logo_uri=draw(urls()),
        )
        for i in range(num_parties)
    ]


@composite
def geopolitical_units(draw: _DrawType):
    """
    Generates a `GeopoliticalUnit` object.
    :param draw: Hidden argument, used by Hypothesis.
    """
    return GeopoliticalUnit(
        object_id=str(draw(uuids())),
        name=draw(emails()),
        type=draw(reporting_unit_types()),
        contact_information=draw(contact_infos()),
    )


@composite
def candidates(draw: _DrawType, party_list: Optional[List[Party]]):
    """
    Generates a `Candidate` object, assigning it one of the parties from `party_list` at random,
    with a chance that there will be no party assigned at all.
    :param draw: Hidden argument, used by Hypothesis.
    :param party_list: A list of `Party` objects. If None, then the resulting `Candidate`
        will have no party.
    """
    if party_list:
        party = party_list[draw(integers(0, len(party_list) - 1))]
        party_id = party.get_party_id()
    else:
        party_id = None

    return Candidate(
        str(draw(uuids())),
        draw(internationalized_human_names()),
        party_id,
        draw(one_of(just(None), urls())),
    )


def _candidate_to_selection_description(
    candidate: Candidate, sequence_order: int
) -> SelectionDescription:
    """
    Given a `Candidate` and its position in a list of candidates, returns an equivalent
    `SelectionDescription`. The selection's `object_id` will contain the candidates's
    `object_id` within, but will have a "c-" prefix attached, so you'll be able to
    tell that they're related.
    """
    return SelectionDescription(
        f"c-{candidate.object_id}", candidate.get_candidate_id(), sequence_order
    )


@composite
def candidate_contest_descriptions(
    draw: _DrawType,
    sequence_order: int,
    party_list: List[Party],
    geo_units: List[GeopoliticalUnit],
    n: Optional[int] = None,
    m: Optional[int] = None,
):
    """
    Generates a tuple: a `List[Candidate]` and a corresponding `CandidateContestDescription` for
    an n-of-m contest.
    :param draw: Hidden argument, used by Hypothesis.
    :param sequence_order: integer describing the order of this contest; make these sequential when
        generating many contests.
    :param party_list: A list of `Party` objects; each candidate's party is drawn at random from this list.
    :param geo_units: A list of `GeopoliticalUnit`; one of these goes into the `electoral_district_id`
    :param n: optional integer, specifying a particular value for n in this n-of-m contest, otherwise
        it's varied by Hypothesis.
    :param m: optional integer, specifying a particular value for m in this n-of-m contest, otherwise
        it's varied by Hypothesis.
    """

    if n is None:
        n = draw(integers(1, 3))
    if m is None:
        m = n + draw(integers(0, 3))  # for an n-of-m election

    party_ids = [p.get_party_id() for p in party_list]

    contest_candidates = draw(lists(candidates(party_list), min_size=m, max_size=m))
    selection_descriptions = [
        _candidate_to_selection_description(contest_candidates[i], i) for i in range(m)
    ]

    vote_variation = VoteVariationType.one_of_m if n == 1 else VoteVariationType.n_of_m

    return (
        contest_candidates,
        CandidateContestDescription(
            object_id=str(draw(uuids())),
            electoral_district_id=geo_units[
                draw(integers(0, len(geo_units) - 1))
            ].object_id,
            sequence_order=sequence_order,
            vote_variation=vote_variation,
            number_elected=n,
            votes_allowed=n,  # should this be None or n?
            name=draw(emails()),
            ballot_selections=selection_descriptions,
            ballot_title=draw(internationalized_texts()),
            ballot_subtitle=draw(internationalized_texts()),
            primary_party_ids=party_ids,
        ),
    )


@composite
def contest_descriptions_room_for_overvoting(
    draw: _DrawType,
    sequence_order: int,
    party_list: List[Party],
    geo_units: List[GeopoliticalUnit],
):
    """
    Similar to `contest_descriptions`, but guarantees that for the n-of-m contest that n < m,
    therefore it's possible to construct an "overvoted" plaintext, which should then fail subsequent tests.
    :param draw: Hidden argument, used by Hypothesis.
    :param sequence_order: integer describing the order of this contest; make these sequential when
        generating many contests.
    :param party_list: A list of `Party` objects; each candidate's party is drawn at random from this list.
    :param geo_units: A list of `GeopoliticalUnit`; one of these goes into the `electoral_district_id`
    """
    n = draw(integers(1, 3))
    m = n + draw(integers(1, 3))
    return draw(
        candidate_contest_descriptions(
            sequence_order=sequence_order,
            party_list=party_list,
            geo_units=geo_units,
            n=n,
            m=m,
        )
    )


@composite
def referendum_contest_descriptions(
    draw: _DrawType, sequence_order: int, geo_units: List[GeopoliticalUnit]
):
    """
    Generates a tuple: a list of party-less candidates and a corresponding `ReferendumContestDescription`.
    :param draw: Hidden argument, used by Hypothesis.
    :param sequence_order: integer describing the order of this contest; make these sequential when
        generating many contests.
    :param geo_units: A list of `GeopoliticalUnit`; one of these goes into the `electoral_district_id`
    """
    n = draw(integers(1, 3))

    contest_candidates = draw(lists(candidates(None), min_size=n, max_size=n))
    selection_descriptions = [
        _candidate_to_selection_description(contest_candidates[i], i) for i in range(n)
    ]

    return (
        contest_candidates,
        ReferendumContestDescription(
            object_id=str(draw(uuids())),
            electoral_district_id=geo_units[
                draw(integers(0, len(geo_units) - 1))
            ].object_id,
            sequence_order=sequence_order,
            vote_variation=VoteVariationType.one_of_m,
            number_elected=1,
            votes_allowed=1,  # should this be None or 1?
            name=draw(emails()),
            ballot_selections=selection_descriptions,
            ballot_title=draw(internationalized_texts()),
            ballot_subtitle=draw(internationalized_texts()),
        ),
    )


@composite
def contest_descriptions(
    draw: _DrawType,
    sequence_order: int,
    party_list: List[Party],
    geo_units: List[GeopoliticalUnit],
):
    """
    Generates either the result of `referendum_contest_descriptions` or `candidate_contest_descriptions`.
    :param draw: Hidden argument, used by Hypothesis.
    :param sequence_order: integer describing the order of this contest; make these sequential when
        generating many contests.
    :param party_list: A list of `Party` objects; each candidate's party is drawn at random from this list.
        See `candidates` for details on this assignment.
    :param geo_units: A list of `GeopoliticalUnit`; one of these goes into the `electoral_district_id`
    """
    return draw(
        one_of(
            referendum_contest_descriptions(sequence_order, geo_units),
            candidate_contest_descriptions(sequence_order, party_list, geo_units),
        )
    )


@composite
def election_descriptions(
    draw: _DrawType, max_num_parties: int = 3, max_num_contests: int = 3
):
    """
    Generates an `ElectionDescription` -- the top-level object describing an election.
    :param draw: Hidden argument, used by Hypothesis.
    :param max_num_parties: The largest number of parties that will be generated (default: 3)
    :param max_num_contests: The largest number of contests that will be generated (default: 3)
    """
    assert max_num_parties > 0, "need at least one party"
    assert max_num_contests > 0, "need at least one contest"

    geo_units = [draw(geopolitical_units())]
    num_parties: int = draw(integers(1, max_num_parties))

    # keep this small so tests run faster
    parties: List[Party] = draw(party_lists(num_parties))
    num_contests: int = draw(integers(1, max_num_contests))

    # generate a collection candidates mapped to contest descritpions
    candidate_contests: List[Tuple[List[Candidate], ContestDescription]] = [
        draw(contest_descriptions(i, parties, geo_units)) for i in range(num_contests)
    ]
    assert len(candidate_contests) > 0

    candidates_ = reduce(
        lambda a, b: a + b,
        [candidate_contest[0] for candidate_contest in candidate_contests],
    )
    contests = [candidate_contest[1] for candidate_contest in candidate_contests]

    styles = [draw(ballot_styles(parties, geo_units))]

    # maybe later on we'll do something more complicated with dates
    start_date = draw(datetimes())
    end_date = start_date

    return ElectionDescription(
        election_scope_id=draw(emails()),
        type=ElectionType.general,  # good enough for now
        start_date=start_date,
        end_date=end_date,
        geopolitical_units=geo_units,
        parties=parties,
        candidates=candidates_,
        contests=contests,
        ballot_styles=styles,
        name=draw(internationalized_texts()),
        contact_information=draw(contact_infos()),
    )


@composite
def plaintext_voted_ballots(
    draw: _DrawType, metadata: InternalElectionDescription, count: int = 1
):
    """
    Given
    """
    if count == 1:
        return draw(plaintext_voted_ballot(metadata))
    ballots: List[PlaintextBallot] = []
    for i in range(count):
        ballots.append(draw(plaintext_voted_ballot(metadata)))
    return ballots


@composite
def plaintext_voted_ballot(draw: _DrawType, metadata: InternalElectionDescription):
    """
    Given an `InternalElectionDescription` object, generates an arbitrary `PlaintextBallot` with the
    choices made randomly.
    :param draw: Hidden argument, used by Hypothesis.
    :param metadata: Any `InternalElectionDescription`
    """

    num_ballot_styles = len(metadata.ballot_styles)
    assert num_ballot_styles > 0, "invalid election with no ballot styles"

    # pick a ballot style at random
    ballot_style = metadata.ballot_styles[draw(integers(0, num_ballot_styles - 1))]

    contests = metadata.get_contests_for(ballot_style.object_id)
    assert len(contests) > 0, "invalid ballot style with no contests in it"

    voted_contests: List[PlaintextBallotContest] = []
    for contest in contests:
        assert contest.is_valid(), "every contest needs to be valid"
        n = contest.number_elected  # we need exactly this many 1's, and the rest 0's
        ballot_selections = contest.ballot_selections
        assert len(ballot_selections) >= n

        random = Random(draw(integers()))
        random.shuffle(ballot_selections)
        cut_point = draw(integers(0, n))
        yes_votes = ballot_selections[:cut_point]
        no_votes = ballot_selections[cut_point:]

        voted_selections = [
            selection_from(description, is_placeholder=False, is_affirmative=True)
            for description in yes_votes
        ] + [
            selection_from(description, is_placeholder=False, is_affirmative=False)
            for description in no_votes
        ]

        voted_contests.append(
            PlaintextBallotContest(contest.object_id, voted_selections)
        )

    return PlaintextBallot(str(draw(uuids())), ballot_style.object_id, voted_contests)


CIPHERTEXT_ELECTIONS_TUPLE_TYPE = Tuple[ElementModQ, CiphertextElectionContext]


@composite
def ciphertext_elections(draw: _DrawType, election_description: ElectionDescription):
    """
    Generates a `CiphertextElectionContext` with a single public-private key pair as the encryption context.

    In a real election, the key ceremony would be used to generate a shared public key.

    :param draw: Hidden argument, used by Hypothesis.
    :param election_description: An `ElectionDescription` object, with which the `CiphertextElectionContext` will be associated
    :return: a tuple of a `CiphertextElectionContext` and the secret key associated with it
    """
    secret_key, public_key = draw(elgamal_keypairs())
    ciphertext_election_with_secret: CIPHERTEXT_ELECTIONS_TUPLE_TYPE = (
        secret_key,
        make_ciphertext_election_context(
            number_of_guardians=1,
            quorum=1,
            elgamal_public_key=public_key,
            description_hash=election_description.crypto_hash(),
        ),
    )
    return ciphertext_election_with_secret


ELECTIONS_AND_BALLOTS_TUPLE_TYPE = Tuple[
    ElectionDescription,
    InternalElectionDescription,
    List[PlaintextBallot],
    ElementModQ,
    CiphertextElectionContext,
]


@composite
def elections_and_ballots(draw: _DrawType, num_ballots: int = 3):
    """
    A convenience generator to generate all of the necessary components for simulating an election.
    Every ballot will match the same ballot style. Hypothesis doesn't
    let us declare a type hint on strategy return values, so you can use `ELECTIONS_AND_BALLOTS_TUPLE_TYPE`.

    :param draw: Hidden argument, used by Hypothesis.
    :param num_ballots: The number of ballots to generate (default: 3).
    :reeturn: a tuple of: an `InternalElectionDescription`, a list of plaintext ballots, an ElGamal secret key,
        and a `CiphertextElectionContext`
    """
    assert num_ballots >= 0, "You're asking for a negative number of ballots?"
    election_description = draw(election_descriptions())
    internal_election_description = InternalElectionDescription(election_description)

    ballots = [
        draw(plaintext_voted_ballots(internal_election_description))
        for _ in range(num_ballots)
    ]

    secret_key, context = draw(ciphertext_elections(election_description))

    mock_election: ELECTIONS_AND_BALLOTS_TUPLE_TYPE = (
        election_description,
        internal_election_description,
        ballots,
        secret_key,
        context,
    )
    return mock_election



from dataclasses import dataclass, field
from typing import Optional

from .ballot import (
    BallotBoxState,
    CiphertextBallot,
    CiphertextAcceptedBallot,
    from_ciphertext_ballot,
)
from .ballot_store import BallotStore

from .election import CiphertextElectionContext, InternalElectionDescription
from .logs import log_warning
from .ballot_validator import ballot_is_valid_for_election


@dataclass
class BallotBox(object):
    """
    A stateful convenience wrapper to cache election data
    """

    _metadata: InternalElectionDescription = field()
    _encryption: CiphertextElectionContext = field()
    _store: BallotStore = field(default_factory=lambda: BallotStore())

    def cast(self, ballot: CiphertextBallot) -> Optional[CiphertextAcceptedBallot]:
        """
        Cast a specific encrypted `CiphertextBallot`
        """
        return accept_ballot(
            ballot, BallotBoxState.CAST, self._metadata, self._encryption, self._store
        )

    def spoil(self, ballot: CiphertextBallot) -> Optional[CiphertextAcceptedBallot]:
        """
        Spoil a specific encrypted `CiphertextBallot`
        """
        return accept_ballot(
            ballot,
            BallotBoxState.SPOILED,
            self._metadata,
            self._encryption,
            self._store,
        )


def accept_ballot(
    ballot: CiphertextBallot,
    state: BallotBoxState,
    metadata: InternalElectionDescription,
    context: CiphertextElectionContext,
    store: BallotStore,
) -> Optional[CiphertextAcceptedBallot]:
    """
    Accept a ballot within the context of a specified election and against an existing data store
    Verified that the ballot is valid for the election `metadata` and `context` and
    that the ballot has not already been cast or spoiled.
    :return: a `CiphertextAcceptedBallot` or `None` if there was an error
    """
    if not ballot_is_valid_for_election(ballot, metadata, context):
        return None

    ballot_exists, existing_ballot = store.exists(ballot.object_id)
    if ballot_exists and existing_ballot is not None:
        log_warning(
            f"error accepting ballot, {ballot.object_id} already exists with state: {existing_ballot.state}"
        )
        return None

    # TODO: ISSUE #56: check if the ballot includes the nonce, and regenerate the proofs
    # TODO: ISSUE #56: check if the ballot includes the proofs, if it does not include the nonce

    ballot_box_ballot = from_ciphertext_ballot(ballot, state)

    store.set(ballot_box_ballot.object_id, ballot_box_ballot)
    return store.get(ballot_box_ballot.object_id)
